@@ -8,17 +8,20 @@
 #include "msfileaccessor.h"
 #include "pwizmsfilereader.h"
 #include "timsmsfilereader.h"
+#include "bafasciifilereader.h"
 #include "xymsfilereader.h"
 
 
 #include "../exception/exceptionnotfound.h"
 #include "../exception/exceptionnotpossible.h"
+#include "../exception/exceptionnotrecognized.h"
 #include "../msrun/msrunid.h"
 #include "../msrun/private/timsframesmsrunreader.h"
 
 #include "../msrun/private/pwizmsrunreader.h"
 #include "../msrun/private/timsmsrunreader.h"
 #include "../msrun/private/timsmsrunreaderms2.h"
+#include "../msrun/bafasciimsrunreader.h"
 #include "../msrun/xymsrunreader.h"
 
 #include "../utils.h"
@@ -36,6 +39,13 @@ MsFileAccessor::MsFileAccessor(const QString &file_name,
   if(!file.exists())
     throw(ExceptionNotFound(QObject::tr("File %1 not found.")
                               .arg(QFileInfo(file_name).absoluteFilePath())));
+
+
+  m_oboPsiModTermNativeIDFormat.m_accession = "MS:1000824";
+  m_oboPsiModTermNativeIDFormat.m_name      = "no nativeID format";
+  m_oboPsiModTermNativeIDFormat.m_definition =
+    "No nativeID format indicates that the file tagged with this term does not "
+    "contain spectra that can have a nativeID format.";
 }
 
 
@@ -45,6 +55,7 @@ MsFileAccessor::MsFileAccessor(const MsFileAccessor &other)
     m_fileFormat(other.m_fileFormat),
     m_fileReaderType(other.m_fileReaderType)
 {
+  m_oboPsiModTermNativeIDFormat = other.m_oboPsiModTermNativeIDFormat;
 }
 
 MsFileAccessor::~MsFileAccessor()
@@ -59,12 +70,126 @@ MsFileAccessor::getFileName() const
 }
 
 
-MzFormat
+MsDataFormat
 MsFileAccessor::getFileFormat() const
 {
   return m_fileFormat;
 }
 
+const OboPsiModTerm
+MsFileAccessor::getOboPsiModTermFileFormat() const
+{
+  OboPsiModTerm term;
+
+  // is_a: MS:1000560 ! mass spectrometer file format
+  switch(m_fileFormat)
+    {
+      case MsDataFormat::abSciexT2D:
+        term.m_accession = "MS:1001560";
+        term.m_name      = "SCIEX TOF/TOF T2D format";
+        term.m_definition =
+          "Applied Biosystems/MDS Analytical Technologies TOF/TOF instrument "
+          "export format.";
+        break;
+      case MsDataFormat::abSciexWiff:
+        term.m_accession  = "MS:1000562";
+        term.m_name       = "ABI WIFF format";
+        term.m_definition = "Applied Biosystems WIFF file format.";
+        break;
+      case MsDataFormat::agilentMassHunter:
+        term.m_accession = "MS:1001509";
+        term.m_name      = "Agilent MassHunter format";
+        term.m_definition =
+          "A data file format found in an Agilent MassHunter directory which "
+          "contains raw data acquired by an Agilent mass spectrometer.";
+        break;
+      case MsDataFormat::brukerBaf:
+        break;
+      case MsDataFormat::brukerFid:
+        term.m_accession  = "MS:1000825";
+        term.m_name       = "Bruker FID format";
+        term.m_definition = "Bruker FID file format.";
+        break;
+      case MsDataFormat::brukerTims:
+        term.m_accession  = "MS:1002817";
+        term.m_name       = "Bruker TDF format";
+        term.m_definition = "Bruker TDF raw file format.";
+        break;
+      case MsDataFormat::brukerYep:
+        term.m_accession  = "MS:1000567";
+        term.m_name       = "Bruker/Agilent YEP format";
+        term.m_definition = "Bruker/Agilent YEP file format.";
+        break;
+      case MsDataFormat::MGF:
+        term.m_accession  = "MS:1001062";
+        term.m_name       = "Mascot MGF format";
+        term.m_definition = "Mascot MGF file format.";
+        break;
+      case MsDataFormat::msn:
+        break;
+      case MsDataFormat::mz5:
+        term.m_accession  = "MS:1001881";
+        term.m_name       = "mz5 format";
+        term.m_definition = "mz5 file format, modelled after mzML.";
+        break;
+      case MsDataFormat::mzML:
+        term.m_accession = "MS:1000584";
+        term.m_name      = "mzML format";
+        term.m_definition =
+          "Proteomics Standards Inititative mzML file format.";
+        break;
+      case MsDataFormat::mzXML:
+        term.m_accession  = "MS:1000566";
+        term.m_name       = "ISB mzXML format";
+        term.m_definition = "Institute of Systems Biology mzXML file format.";
+        break;
+      case MsDataFormat::SQLite3:
+        break;
+      case MsDataFormat::thermoRaw:
+
+        term.m_accession  = "MS:1000563";
+        term.m_name       = "Thermo RAW format";
+        term.m_definition = "Thermo Scientific RAW file format.";
+        break;
+      case MsDataFormat::unknown:
+        break;
+      case MsDataFormat::watersRaw:
+        term.m_accession = "MS:1000526";
+        term.m_name      = "Waters raw format";
+        term.m_definition =
+          "Waters data file format found in a Waters RAW directory, generated "
+          "from an MS acquisition.";
+        break;
+      case MsDataFormat::brukerBafAscii:
+        term.m_accession = "MS:1001369";
+        term.m_name      = "BafAscii text format";
+        term.m_definition =
+          "Simple text file format obtained by exporting Bruker Baf to ascii "
+          "using Bruker software";
+        break;
+      case MsDataFormat::xy:
+        term.m_accession = "MS:1001369";
+        term.m_name      = "text format";
+        term.m_definition =
+          "Simple text file format of \"m/z<separator>intensity\" value pairs "
+          "for a single mass spectrum, a PMF (or single MS2) search.";
+        break;
+      default:
+        break;
+    }
+
+  return term;
+}
+
+
+const OboPsiModTerm &
+MsFileAccessor::getOboPsiModTermNativeIDFormat() const
+{
+
+
+  return m_oboPsiModTermNativeIDFormat;
+}
+
 
 std::vector<MsRunIdCstSPtr>
 MsFileAccessor::getMsRunIds()
@@ -79,7 +204,8 @@ MsFileAccessor::getMsRunIds()
     pwiz_ms_file_reader.getMsRunIds(m_xmlPrefix);
   if(ms_run_ids.size())
     {
-      // qDebug() << "Might well be handled using the Pwiz code.";
+      qDebug() << "Might well be handled using the Pwiz code.";
+
       m_fileFormat     = pwiz_ms_file_reader.getFileFormat();
       m_fileReaderType = FileReaderType::pwiz;
 
@@ -94,7 +220,7 @@ MsFileAccessor::getMsRunIds()
       return ms_run_ids;
     }
 
-  // qDebug() << "The Pwiz reader did not work.";
+  qDebug() << "The Pwiz reader did not work.";
 
   // Try the TimsData reader
 
@@ -110,7 +236,7 @@ MsFileAccessor::getMsRunIds()
 
   if(ms_run_ids.size())
     {
-      // qDebug() << "Might well be handled using the Bruker code";
+      qDebug() << "Might well be handled using the Bruker code";
 
       m_fileName       = tims_dir;
       m_fileFormat     = tims_file_reader.getFileFormat();
@@ -122,14 +248,49 @@ MsFileAccessor::getMsRunIds()
           m_fileReaderType = pref->second;
         }
 
-      // qDebug() << "Returning Bruker::tims ms run(s)."
-      //          << "with preferred reader type:"
-      //          << Utils::fileReaderTypeAsString(m_fileReaderType);
+      qDebug() << "Returning Bruker::tims ms run(s)."
+               << "with preferred reader type:"
+               << Utils::fileReaderTypeAsString(m_fileReaderType);
 
       return ms_run_ids;
     }
 
-  // qDebug() << "The Tims reader did not work.";
+  qDebug() << "The Tims reader did not work.";
+
+  // Try the Baf->ascii export format from Bruker Compass
+
+  try
+    {
+      ms_run_ids.clear();
+      BafAsciiFileReader baf_ascii_ms_file_reader(m_fileName);
+
+      ms_run_ids = baf_ascii_ms_file_reader.getMsRunIds(m_xmlPrefix);
+
+      if(ms_run_ids.size())
+        {
+          qDebug() << "Might well be handled using the BafAscii code";
+
+          m_fileReaderType = FileReaderType::bafascii;
+
+          m_fileFormat = baf_ascii_ms_file_reader.getFileFormat();
+
+          if(m_fileFormat == MsDataFormat::unknown)
+            {
+              ms_run_ids.clear();
+            }
+          else
+            {
+              return ms_run_ids;
+            }
+        }
+    }
+  catch(const pappso::PappsoException &error)
+    {
+      qDebug() << "This is not a BafAscii code file" << error.qwhat();
+    }
+
+
+  qDebug() << "The BafAscii reader did not work.";
 
   // At this point try the XyMsFileReader
 
@@ -139,7 +300,8 @@ MsFileAccessor::getMsRunIds()
 
   if(ms_run_ids.size())
     {
-      // qDebug() << "Might well be handled using the XY code";
+      qDebug() << "Might well be handled using the XY code";
+
       m_fileReaderType = FileReaderType::xy;
 
       m_fileFormat = xy_ms_file_reader.getFileFormat();
@@ -147,18 +309,20 @@ MsFileAccessor::getMsRunIds()
       return ms_run_ids;
     }
 
-  // qDebug() << "The XY reader did not work.";
+  qDebug() << "The XY reader did not work.";
 
   return ms_run_ids;
 }
 
 
 void
-MsFileAccessor::setPreferredFileReaderType(MzFormat format,
+MsFileAccessor::setPreferredFileReaderType(MsDataFormat format,
                                            FileReaderType reader_type)
 {
+  // qDebug();
+
   auto ret = m_preferredFileReaderTypeMap.insert(
-    std::pair<MzFormat, FileReaderType>(format, reader_type));
+    std::pair<MsDataFormat, FileReaderType>(format, reader_type));
 
   if(!ret.second)
     {
@@ -169,8 +333,10 @@ MsFileAccessor::setPreferredFileReaderType(MzFormat format,
 
 
 FileReaderType
-MsFileAccessor::getpreferredFileReaderType(MzFormat format)
+MsFileAccessor::getpreferredFileReaderType(MsDataFormat format)
 {
+  // qDebug();
+
   auto ret = m_preferredFileReaderTypeMap.find(format);
 
   if(ret != m_preferredFileReaderTypeMap.end())
@@ -190,26 +356,16 @@ MsFileAccessor::getFileReaderType() const
 
 
 void
-MsFileAccessor::setSelectedMsRunIdIndex(std::size_t index)
+MsFileAccessor::setSelectedMsRunId(MsRunIdCstSPtr ms_run_id_csp)
 {
-  m_selectedMsRunIdIndex = index;
+  mcsp_selectedMsRunId = ms_run_id_csp;
 }
 
 
-std::size_t
-MsFileAccessor::getSelectedMsRunIdIndex() const
+MsRunIdCstSPtr
+MsFileAccessor::getSelectedMsRunId() const
 {
-  return m_selectedMsRunIdIndex;
-}
-
-
-MsRunIdCstSPtr 
-MsFileAccessor::getSelectedMsRunId()
-{
-  if(m_selectedMsRunIdIndex >= getMsRunIds().size())
-    throw PappsoException(QObject::tr("MsRunId request out-of-bound error."));
-  
-  return getMsRunIds().at(m_selectedMsRunIdIndex);
+  return mcsp_selectedMsRunId;
 }
 
 TimsMsRunReaderMs2SPtr
@@ -247,6 +403,8 @@ MsFileAccessor::buildTimsMsRunReaderMs2SPtr()
 MsRunReaderSPtr
 MsFileAccessor::msRunReaderSPtr(MsRunIdCstSPtr ms_run_id)
 {
+  // qDebug();
+
   // We want to return a MsRunReader that accounts for the configuration that
   // the user might have set.
 
@@ -258,8 +416,10 @@ MsFileAccessor::msRunReaderSPtr(MsRunIdCstSPtr ms_run_id)
   if(getpreferredFileReaderType(m_fileFormat) == FileReaderType::pwiz)
     {
       // qDebug() << "Returning a PwizMsRunReader.";
-
-      return std::make_shared<PwizMsRunReader>(ms_run_id);
+      auto pwiz_reader = std::make_shared<PwizMsRunReader>(ms_run_id);
+      m_oboPsiModTermNativeIDFormat =
+        pwiz_reader->getOboPsiModTermNativeIDFormat();
+      return pwiz_reader;
     }
   else if(getpreferredFileReaderType(m_fileFormat) == FileReaderType::xy)
     {
@@ -286,15 +446,24 @@ MsFileAccessor::msRunReaderSPtr(MsRunIdCstSPtr ms_run_id)
 
       return std::make_shared<TimsMsRunReaderMs2>(ms_run_id);
     }
-  if(m_fileFormat == MzFormat::unknown)
+  else if(getpreferredFileReaderType(m_fileFormat) == FileReaderType::bafascii)
     {
-      if(ms_run_id.get()->getMzFormat() == MzFormat::xy)
+      // qDebug() << "Returning a BafAsciiMsRunReader.";
+
+      return std::make_shared<BafAsciiMsRunReader>(ms_run_id);
+    }
+  if(m_fileFormat == MsDataFormat::unknown)
+    {
+      if(ms_run_id.get()->getMsDataFormat() == MsDataFormat::xy)
         {
           return std::make_shared<XyMsRunReader>(ms_run_id);
         }
       else
         {
-          return std::make_shared<PwizMsRunReader>(ms_run_id);
+          auto pwiz_reader = std::make_shared<PwizMsRunReader>(ms_run_id);
+          m_oboPsiModTermNativeIDFormat =
+            pwiz_reader->getOboPsiModTermNativeIDFormat();
+          return pwiz_reader;
         }
     }
   else
@@ -306,25 +475,23 @@ MsFileAccessor::msRunReaderSPtr(MsRunIdCstSPtr ms_run_id)
 }
 
 
-MsRunReaderSPtr 
+MsRunReaderSPtr
 MsFileAccessor::msRunReaderSPtr(std::size_t ms_run_id_index)
 {
   std::vector<MsRunIdCstSPtr> ms_run_ids = getMsRunIds();
   if(ms_run_id_index >= ms_run_ids.size())
     throw PappsoException(QObject::tr("MsRunId request out-of-bound error."));
-  
+
   return msRunReaderSPtr(ms_run_ids.at(ms_run_id_index));
 }
 
 
-MsRunReaderSPtr 
-MsFileAccessor::msRunReaderSPtrForSelectedMsRunIdIndex()
+MsRunReaderSPtr
+MsFileAccessor::msRunReaderSPtrForSelectedMsRunId()
 {
-  std::vector<MsRunIdCstSPtr> ms_run_ids = getMsRunIds();
-  if(m_selectedMsRunIdIndex >= ms_run_ids.size())
-    throw PappsoException(QObject::tr("MsRunId request out-of-bound error."));
-  
-  return msRunReaderSPtr(ms_run_ids.at(m_selectedMsRunIdIndex));
+  // qDebug();
+
+  return msRunReaderSPtr(mcsp_selectedMsRunId);
 }
 
 
@@ -344,22 +511,28 @@ MsFileAccessor::buildMsRunReaderSPtr(
       QObject::tr("unable to build a reader : file %1 not found.")
         .arg(QFileInfo(ms_run_id.get()->getFileName()).absoluteFilePath())));
 
-  MzFormat file_format = ms_run_id.get()->getMzFormat();
+  MsDataFormat file_format = ms_run_id.get()->getMsDataFormat();
 
-  if(file_format == MzFormat::xy)
+  if(file_format == MsDataFormat::xy)
     {
       // qDebug() << "Returning a XyMsRunReader.";
 
       return std::make_shared<XyMsRunReader>(ms_run_id);
     }
-  else if(file_format == MzFormat::unknown)
+  else if(file_format == MsDataFormat::brukerBafAscii)
+    {
+      // qDebug() << "Returning a XyMsRunReader.";
+
+      return std::make_shared<BafAsciiMsRunReader>(ms_run_id);
+    }
+  else if(file_format == MsDataFormat::unknown)
     {
       throw(PappsoException(
         QObject::tr("unable to build a reader for %1 : unknown file format")
           .arg(QFileInfo(ms_run_id.get()->getFileName()).absoluteFilePath())));
     }
 
-  else if(file_format == MzFormat::brukerTims)
+  else if(file_format == MsDataFormat::brukerTims)
     {
       if(preferred_file_reader_type == pappso::FileReaderType::tims)
         {
@@ -369,14 +542,18 @@ MsFileAccessor::buildMsRunReaderSPtr(
         {
           return std::make_shared<TimsMsRunReaderMs2>(ms_run_id);
         }
-
+      else if(preferred_file_reader_type == pappso::FileReaderType::tims_frames)
+        {
+          qDebug()
+            << "returning std::make_shared<TimsFramesMsRunReader>(ms_run_id).";
+          return std::make_shared<TimsFramesMsRunReader>(ms_run_id);
+        }
       // qDebug() << "by default, build a TimsMsRunReader.";
       return std::make_shared<TimsMsRunReader>(ms_run_id);
     }
   else
     {
       // qDebug() << "Returning a PwizMsRunReader .";
-
       return std::make_shared<PwizMsRunReader>(ms_run_id);
     }
 }
