@@ -11,6 +11,8 @@
 #include "../pappsoexception.h"
 #include "../exception/exceptionnotpossible.h"
 
+#include <QVariant>
+
 
 namespace pappso
 {
@@ -132,10 +134,67 @@ MsRunDataSetTree::addMassSpectrum(
   documentNodeInDtRtMap(
     mass_spectrum_csp->getRtInMinutes(), new_node_p, DataKind::rt);
 
-  // Likewise for the drift time.
+  // Likewise for the ion mobility time.
 
-  documentNodeInDtRtMap(
-    mass_spectrum_csp->getDtInMilliSeconds(), new_node_p, DataKind::dt);
+  double ion_mobility_value = -1;
+
+  MsDataFormat file_format = mcsp_msRunId->getMsDataFormat();
+  bool ok                  = false;
+
+  if(file_format == MsDataFormat::brukerTims)
+    {
+      // Start by looking if there is a OneOverK0 valid value, which
+      // means we have effectively handled a genuine mobility scan spectrum.
+      QVariant ion_mobility_variant_value =
+        mass_spectrum_csp->getParameterValue(
+          QualifiedMassSpectrumParameter::TimsIonMobScanOneOverK0);
+
+      if(ion_mobility_variant_value.isValid())
+        {
+          // Yes, genuine ion mobility scan handled here.
+
+          ion_mobility_value = ion_mobility_variant_value.toDouble(&ok);
+
+          if(!ok)
+            {
+              qFatal(
+                "The data are Bruker timsTOF data but failed to convert valid "
+                "QVariant 1/K0 value to double.");
+            }
+        }
+      else
+        {
+          // We are not handling a genuine single ion mobility scan here.
+          // We must be handling a mass spectrum that correspond to
+          // the combination of all the ion mobility scans for a single
+          // frame. This is when the user asks that ion mobility scans
+          // be flattended. In this case, the OneOverK0 value is not valid
+          // but there are two values that are set:
+          // TimsFrameInvKoBegin and TimsFrameInvKoEnd.
+          // See TimsFramesMsRunReader::readSpectrumCollection2.
+
+          // Test one of these values as a sanity check. But
+          // give the value of -1 for the ion mobility because we do not
+          // have ion mobility data here.
+
+          ion_mobility_variant_value = mass_spectrum_csp->getParameterValue(
+            QualifiedMassSpectrumParameter::TimsFrameIonMobOneOverK0Begin);
+
+          if(!ion_mobility_variant_value.isValid())
+            {
+              qFatal(
+                "The data are Bruker timsTOF data but failed to get correct "
+                "ion mobility data. Inconsistency found.");
+            }
+        }
+    }
+  else
+    {
+      ion_mobility_value = mass_spectrum_csp->getDtInMilliSeconds();
+    }
+
+  if(ion_mobility_value != -1)
+    documentNodeInDtRtMap(ion_mobility_value, new_node_p, DataKind::dt);
 
   ++m_spectrumCount;
 
@@ -541,10 +600,67 @@ MsRunDataSetTree::addMassSpectrum(
   documentNodeInDtRtMap(
     mass_spectrum_csp->getRtInMinutes(), new_node_p, DataKind::rt);
 
-  // Likewise for the drift time.
+  // Likewise for the ion mobility time.
 
-  documentNodeInDtRtMap(
-    mass_spectrum_csp->getDtInMilliSeconds(), new_node_p, DataKind::dt);
+  double ion_mobility_value = -1;
+
+  MsDataFormat file_format = mcsp_msRunId->getMsDataFormat();
+  bool ok                  = false;
+
+  if(file_format == MsDataFormat::brukerTims)
+    {
+      // Start by looking if there is a OneOverK0 valid value, which
+      // means we have effectively handled a genuine mobility scan spectrum.
+      QVariant ion_mobility_variant_value =
+        mass_spectrum_csp->getParameterValue(
+          QualifiedMassSpectrumParameter::TimsIonMobScanOneOverK0);
+
+      if(ion_mobility_variant_value.isValid())
+        {
+          // Yes, genuine ion mobility scan handled here.
+
+          ion_mobility_value = ion_mobility_variant_value.toDouble(&ok);
+
+          if(!ok)
+            {
+              qFatal(
+                "The data are Bruker timsTOF data but failed to convert valid "
+                "QVariant 1/K0 value to double.");
+            }
+        }
+      else
+        {
+          // We are not handling a genuine single ion mobility scan here.
+          // We must be handling a mass spectrum that correspond to
+          // the combination of all the ion mobility scans for a single
+          // frame. This is when the user asks that ion mobility scans
+          // be flattended. In this case, the OneOverK0 value is not valid
+          // but there are two values that are set:
+          // TimsFrameInvKoBegin and TimsFrameInvKoEnd.
+          // See TimsFramesMsRunReader::readSpectrumCollection2.
+
+          // Test one of these values as a sanity check. But
+          // give the value of -1 for the ion mobility because we do not
+          // have ion mobility data here.
+
+          ion_mobility_variant_value = mass_spectrum_csp->getParameterValue(
+            QualifiedMassSpectrumParameter::TimsFrameIonMobOneOverK0Begin);
+
+          if(!ion_mobility_variant_value.isValid())
+            {
+              qFatal(
+                "The data are Bruker timsTOF data but failed to get correct "
+                "ion mobility data. Inconsistency found.");
+            }
+        }
+    }
+  else
+    {
+      ion_mobility_value = mass_spectrum_csp->getDtInMilliSeconds();
+    }
+
+  if(ion_mobility_value != -1)
+    documentNodeInDtRtMap(ion_mobility_value, new_node_p, DataKind::dt);
 
   // qDebug() << "New index/node map:"
   //<< mass_spectrum_csp->getMassSpectrumId().getSpectrumIndex() << "/"
@@ -855,7 +971,8 @@ MsRunDataSetTree::addDataSetQualMassSpectraInsideDtRtRange(
   for(MapIterator iterator = start_iterator; iterator != end_iterator;
       ++iterator)
     {
-      // We are iterating in MapPair items.
+      // We are iterating in DoubleNodeVectorMap MapPair items,
+      // with double = rt or dt and NodeVector being just that.
 
       NodeVector node_vector = iterator->second;
 
@@ -869,7 +986,7 @@ MsRunDataSetTree::addDataSetQualMassSpectraInsideDtRtRange(
 
 #if 0
           // Sanity check only for deep debugging.
-          
+
           if(qualified_mass_spectrum_csp == nullptr ||
              qualified_mass_spectrum_csp.get() == nullptr)
             {
