@@ -27,6 +27,8 @@
 
 #include "timsframesmsrunreader.h"
 #include "../../exception/exceptionnotimplemented.h"
+#include "../../exception/exceptioninterrupted.h"
+#include "../../exception/exceptionnotpossible.h"
 #include <QDebug>
 
 using namespace pappso;
@@ -102,7 +104,8 @@ void
 TimsFramesMsRunReader::readSpectrumCollection(
   SpectrumCollectionHandlerInterface &handler)
 {
-  qDebug() << "Reading the spectrum collection with no specific configuration.";
+  // qDebug() << "Reading the spectrum collection with no specific
+  // configuration.";
   MsRunReadConfig config;
   readSpectrumCollection2(config, handler);
 }
@@ -112,119 +115,147 @@ void
 TimsFramesMsRunReader::readSpectrumCollection2(
   const MsRunReadConfig &config, SpectrumCollectionHandlerInterface &handler)
 {
-  qDebug().noquote() << "20231129 Reading the spectrum collection with this "
-                        "specific configuration:"
-                     << config.toString();
-
-  // We want to restrict the data reading process to the configuration provided
-  // as parameter.
-  // The frame record subset will include all frames fulfiling the config
-  // requirements (retention time and ms levels)
+  // qDebug().noquote() << "Reading the spectrum collection with this "
+  //                       "specific configuration:"
+  //                    << config.toString();
 
   std::vector<std::size_t> subset_of_tims_frame_ids;
 
-  // The very first sorting of frames is perfomed on the basis of the
-  // retention time range that the user might have defined. Otherwise, all the
-  // frames are deemed suitable for next processing steps.
 
-  // Then, the MS level is checked against the config provided by the user.
+  bool asked_ion_mobility_scan_num_range = false;
+
+  quint32 mobility_scan_num_range_begin = std::numeric_limits<quint32>::max();
+  quint32 mobility_scan_num_range_end   = std::numeric_limits<quint32>::max();
+  quint32 mobility_scan_num_range_width = std::numeric_limits<quint32>::max();
+
+  double mobility_one_over_k0             = std::numeric_limits<double>::max();
+  double mobility_one_over_k0_range_begin = std::numeric_limits<double>::max();
+  double mobility_one_over_k0_range_end   = std::numeric_limits<double>::max();
+
+  if(!config
+        .getParameterValue(
+          MsRunReadConfigParameter::TimsFrameIonMobScanIndexBegin)
+        .isNull() &&
+     !config
+        .getParameterValue(
+          MsRunReadConfigParameter::TimsFrameIonMobScanIndexEnd)
+        .isNull())
+    {
+      mobility_scan_num_range_begin =
+        config
+          .getParameterValue(
+            MsRunReadConfigParameter::TimsFrameIonMobScanIndexBegin)
+          .toUInt();
+      mobility_scan_num_range_end =
+        config
+          .getParameterValue(
+            MsRunReadConfigParameter::TimsFrameIonMobScanIndexEnd)
+          .toUInt();
+
+      // We need the range width below.
+      mobility_scan_num_range_width =
+        mobility_scan_num_range_end + 1 - mobility_scan_num_range_begin;
+
+      asked_ion_mobility_scan_num_range = true;
+
+      // Be sure to check in the frames loop below that the user might
+      // have asked for an ion mobility range but on the basis of the 1/K0 unit.
+    }
+
+  const std::vector<FrameIdDescr> &frame_id_descr_list =
+    msp_timsData->getFrameIdDescrList();
+
+  // Just for the feedback to the user.
+  std::size_t scan_count = 0;
+
   for(auto const &frame_record : msp_timsData->getTimsFrameRecordList())
     {
+      if(handler.shouldStop())
+        {
+          // qDebug() << "The operation was cancelled. Breaking the loop.";
+          throw ExceptionInterrupted(
+            QObject::tr("Reading timsTOF data cancelled by the user."));
+        }
+
       if(frame_record.frame_id == 0)
         continue;
 
-      if(config.acceptRetentionTimeInSeconds(frame_record.frame_time))
+      if(!config.acceptRetentionTimeInSeconds(frame_record.frame_time))
+        continue;
+
+      std::size_t ms_level = 2;
+      if(frame_record.msms_type == 0)
+        ms_level = 1;
+
+      if(!config.acceptMsLevel(ms_level))
+        continue;
+
+      subset_of_tims_frame_ids.push_back(frame_record.frame_id);
+
+      if(mobility_scan_num_range_width != std::numeric_limits<int>::max())
         {
-          std::size_t ms_level = 2;
-          if(frame_record.msms_type == 0)
-            ms_level = 1;
-          if(config.acceptMsLevel(ms_level))
-            {
-              subset_of_tims_frame_ids.push_back(frame_record.frame_id);
-            }
+          scan_count += mobility_scan_num_range_width;
+        }
+      else
+        {
+          scan_count += frame_id_descr_list[frame_record.frame_id].m_size;
         }
     }
 
   // At this point, we have a subset of frame records.
-  std::size_t frames_count = subset_of_tims_frame_ids.size();
-  qDebug() << "The number of retained RT range-matching frames:"
-           << frames_count;
+  std::size_t frame_count = subset_of_tims_frame_ids.size();
+  qDebug() << "The number of retained RT range- and MS level-matching frames : "
+           << frame_count;
 
   // Inform the handler of the spectrum list so that it can handle feedback to
   // the user.
-  handler.spectrumListHasSize(frames_count);
 
-  // Check for ion mobility range :
-  bool asked_ion_mobility_range   = false;
-  quint32 mobility_scan_num_begin = 0;
-  quint32 mobility_scan_num_end   = 0;
-
-  if(!config
-        .getParameterValue(
-          MsRunReadConfigParameter::TimsFramesMsRunReader_mobility_index_begin)
-        .isNull())
-    {
-      asked_ion_mobility_range = true;
-
-      mobility_scan_num_begin =
-        config
-          .getParameterValue(MsRunReadConfigParameter::
-                               TimsFramesMsRunReader_mobility_index_begin)
-          .toUInt();
-      mobility_scan_num_end =
-        config
-          .getParameterValue(
-            MsRunReadConfigParameter::TimsFramesMsRunReader_mobility_index_end)
-          .toUInt();
-    }
-
-  // Check for m/z resolution downgrading
-  std::size_t mz_index_merge_window = 0;
-  if(!config
-        .getParameterValue(
-          MsRunReadConfigParameter::TimsFramesMsRunReader_mz_index_merge_window)
-        .isNull())
-    {
-      mz_index_merge_window =
-        config
-          .getParameterValue(MsRunReadConfigParameter::
-                               TimsFramesMsRunReader_mz_index_merge_window)
-          .toUInt();
-    }
+  // FIXME:
+  // Either we document the number of frames (because we assume we will
+  // flatten them all)...
+  handler.spectrumListHasSize(frame_count);
+  // Or we document the number of actual scans because we might not flatten
+  // all the frames.
+  // handler.spectrumListHasSize(scan_count);
 
   // Check for m/z range selection
   bool asked_mz_range   = false;
   double mz_range_begin = -1;
   double mz_range_end   = -1;
 
-  if(!config
-        .getParameterValue(
-          MsRunReadConfigParameter::TimsFramesMsRunReader_mz_begin)
-        .isNull())
+  if(!config.getParameterValue(MsRunReadConfigParameter::MzRangeBegin).isNull())
     {
       asked_mz_range = true;
 
       mz_range_begin =
-        config
-          .getParameterValue(
-            MsRunReadConfigParameter::TimsFramesMsRunReader_mz_begin)
+        config.getParameterValue(MsRunReadConfigParameter::MzRangeBegin)
+          .toDouble();
+      mz_range_end =
+        config.getParameterValue(MsRunReadConfigParameter::MzRangeEnd)
           .toDouble();
-      mz_range_end = config
-                       .getParameterValue(
-                         MsRunReadConfigParameter::TimsFramesMsRunReader_mz_end)
-                       .toDouble();
 
-      qDebug() << "The m/z range asked is: " << mz_range_begin << "--"
-               << mz_range_end;
+      // qDebug() << "The m/z range asked is: " << mz_range_begin << "--"
+      //          << mz_range_end;
     }
 
-  // Now that we have gathered all the configuration bits from the user,
-  // iterate in the vector of frames records and for each frame ask that
-  // all its scans be combined into a single mass spectrum, by taking into
-  // account the configuration bits provided by the user.
+  // Check for m/z resolution downgrading
+  // The idea is that we merge a number of mz indices into a single index,
+  // which is essentially an increase of the m/z bin size, and therefore
+  // of the resolution/definition of the mass spectrum.
+  std::size_t mz_index_merge_window = 0;
+  if(!config
+        .getParameterValue(
+          MsRunReadConfigParameter::TimsFrameMzIndexMergeWindow)
+        .isNull())
+    {
+      mz_index_merge_window =
+        config
+          .getParameterValue(
+            MsRunReadConfigParameter::TimsFrameMzIndexMergeWindow)
+          .toUInt();
 
-  const std::vector<FrameIdDescr> &frame_id_descr_list =
-    msp_timsData->getFrameIdDescrList();
+      // qDebug() << "mz_index_merge_window=" << mz_index_merge_window;
+    }
 
   // The scan index is the index of the scan in the *whole* mass data file, it
   // is a sequential number of scans over all the frames.
@@ -232,91 +263,100 @@ TimsFramesMsRunReader::readSpectrumCollection2(
 
   for(std::size_t tims_frame_id : subset_of_tims_frame_ids)
     {
+      if(handler.shouldStop())
+        {
+          // qDebug() << "The operation was cancelled. Breaking the loop.";
+          throw ExceptionInterrupted(
+            QObject::tr("Reading timsTOF data cancelled by the user."));
+        }
+
+      // qDebug() << "tims_frame_id=" << tims_frame_id;
+
       const FrameIdDescr &current_frame_record =
         frame_id_descr_list[tims_frame_id];
+      // qDebug() << "tims_frame_id=" << tims_frame_id;
 
       scan_index = current_frame_record.m_cumulSize;
+
       TimsFrameCstSPtr tims_frame_csp =
         msp_timsData->getTimsFrameCstSPtrCached(tims_frame_id);
 
-      // If the user wants to select specific 1/Ko values, we need to
+      // If the user wants to select 1/Ko values in a given range, we need to
       // compute the ion mobility scan value starting from that 1/Ko value in
-      // each frame.
+      // *each* frame. Note that the computed mobility_scan_num_begin and
+      // mobility_scan_num_end would override thoses possibly set with
+      // TimsFramesMsRunReader_mobility_index_begin/end above.
 
       if(!config
             .getParameterValue(
-              MsRunReadConfigParameter::
-                TimsFramesMsRunReader_mobility_one_over_k0_begin)
+              MsRunReadConfigParameter::TimsFrameIonMobOneOverK0Begin)
+            .isNull() &&
+         !config
+            .getParameterValue(
+              MsRunReadConfigParameter::TimsFrameIonMobOneOverK0End)
             .isNull())
         {
-          asked_ion_mobility_range = true;
-
-          mobility_scan_num_begin =
-            tims_frame_csp.get()->getScanNumFromOneOverK0(
-              config
-                .getParameterValue(
-                  MsRunReadConfigParameter::
-                    TimsFramesMsRunReader_mobility_one_over_k0_begin)
-                .toDouble());
-
-          mobility_scan_num_end = tims_frame_csp.get()->getScanNumFromOneOverK0(
+          mobility_one_over_k0_range_begin =
             config
               .getParameterValue(
-                MsRunReadConfigParameter::
-                  TimsFramesMsRunReader_mobility_one_over_k0_end)
-              .toDouble());
+                MsRunReadConfigParameter::TimsFrameIonMobOneOverK0Begin)
+              .toDouble();
+
+          mobility_one_over_k0_range_end =
+            config
+              .getParameterValue(
+                MsRunReadConfigParameter::TimsFrameIonMobOneOverK0End)
+              .toDouble();
+
+          mobility_scan_num_range_begin =
+            tims_frame_csp.get()->getScanNumFromOneOverK0(
+              mobility_one_over_k0_range_begin);
+
+          mobility_scan_num_range_end =
+            tims_frame_csp.get()->getScanNumFromOneOverK0(
+              mobility_one_over_k0_range_end);
+
+          asked_ion_mobility_scan_num_range = true;
         }
 
-      // Get to know the size of the frame, that is,
-      // the number of mobility scans in it.
-      quint32 mobility_scans_count = tims_frame_csp->getTotalNumberOfScans();
+      // Now that we know if the user has asked for an ion mobility range,
+      // either using scan indices or 1/K0 values, we need to double check the
+      // range borders.
+      quint32 count_of_mobility_scans = tims_frame_csp->getTotalNumberOfScans();
 
-      // Provide these two variables for the function below to fill in the
-      // values.
-      quint32 min_mz_index_out = 0;
-      quint32 max_mz_index_out = 0;
-
-      Trace spectrum;
-
-      if(config.needPeakList())
+      if(asked_ion_mobility_scan_num_range)
         {
-          if(asked_ion_mobility_range)
+          if(mobility_scan_num_range_end > (count_of_mobility_scans - 1))
             {
-              if(mobility_scan_num_end > (mobility_scans_count - 1))
-                {
-                  mobility_scan_num_end = mobility_scans_count - 1;
-                }
-            }
-          else
-            {
-              mobility_scan_num_begin = 0;
-              mobility_scan_num_end   = mobility_scans_count - 1;
-            }
-
-          if(asked_mz_range)
-            {
-              spectrum = tims_frame_csp->cumulateScansToTraceMzDownResolution2(
-                mz_index_merge_window,
-                mz_range_begin,
-                mz_range_end,
-                mobility_scan_num_begin,
-                mobility_scan_num_end,
-                min_mz_index_out,
-                max_mz_index_out);
-            }
-          else
-            {
-              spectrum = tims_frame_csp->cumulateScansToTraceMzDownResolution(
-                mz_index_merge_window,
-                mobility_scan_num_begin,
-                mobility_scan_num_end,
-                min_mz_index_out,
-                max_mz_index_out);
+              mobility_scan_num_range_end = count_of_mobility_scans - 1;
             }
         }
+      else
+        {
+          mobility_scan_num_range_begin = 0;
+          mobility_scan_num_range_end   = count_of_mobility_scans - 1;
+        }
 
-      // qDebug() << "Got min_mz_index_out:" << min_mz_index_out;
-      // qDebug() << "Got max_mz_index_out:" << max_mz_index_out;
+      // Now that we know the mobility index range, if we did not set the
+      // mobility one over K0 because that was not the unit used by
+      // the caller, then we can compute these values and set them
+      // later to the qualified mass spectrum parameters.
+      if(mobility_one_over_k0_range_begin == std::numeric_limits<double>::max())
+        mobility_one_over_k0_range_begin =
+          tims_frame_csp->getOneOverK0Transformation(
+            mobility_scan_num_range_begin);
+      if(mobility_one_over_k0_range_end == std::numeric_limits<double>::max())
+        mobility_one_over_k0_range_end =
+          tims_frame_csp->getOneOverK0Transformation(
+            mobility_scan_num_range_end);
+
+      mobility_scan_num_range_width =
+        mobility_scan_num_range_end + 1 - mobility_scan_num_range_begin;
+
+      // We want to provide the inverse mobility for the scan that sits in the
+      // middle of the defined range or the whole range if none is defined..
+      mobility_one_over_k0 = tims_frame_csp.get()->getScanNumFromOneOverK0(
+        mobility_scan_num_range_begin + (mobility_scan_num_range_width / 2));
 
       // Now, with or without the peak list, we have to craft a qualified mass
       // spectrum that will hold all the data about the data in it.
@@ -329,11 +369,12 @@ TimsFramesMsRunReader::readSpectrumCollection2(
 
       // Can be modified to add bits that might help our case
       spectrum_id.setNativeId(
-        QString("frame id=%1 scan index=%2 im_begin=%3 im_end=%4")
+        QString("frame_id=%1 global_scan_index=%2 im_scan_range_begin=%3 "
+                "im_scan_range_end=%4")
           .arg(tims_frame_id)
           .arg(scan_index)
-          .arg(mobility_scan_num_begin)
-          .arg(mobility_scan_num_end));
+          .arg(mobility_scan_num_range_begin)
+          .arg(mobility_scan_num_range_end));
 
       mass_spectrum.setMassSpectrumId(spectrum_id);
 
@@ -345,38 +386,102 @@ TimsFramesMsRunReader::readSpectrumCollection2(
       unsigned int frame_ms_level = tims_frame_csp.get()->getMsLevel();
       mass_spectrum.setMsLevel(frame_ms_level);
 
-      // We cannot set this value because the inverse mobility (1/Ko) measured
-      // by Bruker does not correspond formally to a drift time.
-      mass_spectrum.setDtInMilliSeconds(-1);
+      // Arrival time at half the range.
+      mass_spectrum.setDtInMilliSeconds(tims_frame_csp.get()->getDriftTime(
+        mobility_scan_num_range_begin + (mobility_scan_num_range_width / 2)));
+
+      // 1/K0
+      qDebug() << "mobility_one_over_k0:" << mobility_one_over_k0
+               << "mobility_one_over_k0_range_begin:"
+               << mobility_one_over_k0_range_begin
+               << "mobility_one_over_k0_range_end"
+               << mobility_one_over_k0_range_end;
+
+      if(mobility_one_over_k0 == std::numeric_limits<double>::max() ||
+         mobility_one_over_k0_range_begin ==
+           std::numeric_limits<double>::max() ||
+         mobility_one_over_k0_range_end == std::numeric_limits<double>::max())
+        throw(
+          ExceptionNotPossible("Not possible that mobility_one_over_k0 and its "
+                               "range are undefined."));
+
+      mass_spectrum.setParameterValue(
+        QualifiedMassSpectrumParameter::TimsIonMobScanOneOverK0,
+        mobility_one_over_k0);
+      mass_spectrum.setParameterValue(
+        QualifiedMassSpectrumParameter::TimsFrameIonMobOneOverK0Begin,
+        mobility_one_over_k0_range_begin);
+      mass_spectrum.setParameterValue(
+        QualifiedMassSpectrumParameter::TimsFrameIonMobOneOverK0End,
+        mobility_one_over_k0_range_end);
+
+      // qDebug() << "mobility_scan_num_range_begin:"
+      //          << mobility_scan_num_range_begin
+      //          << "mobility_scan_num_range_end:" <<
+      //          mobility_scan_num_range_end;
+
+      if(mobility_scan_num_range_begin == std::numeric_limits<quint32>::max() ||
+         mobility_scan_num_range_end == std::numeric_limits<quint32>::max())
+        throw(ExceptionNotPossible(
+          "Not possible that mobility_scan_num_range values are undefined."));
+
+      mass_spectrum.setParameterValue(
+        QualifiedMassSpectrumParameter::TimsIonMobScanIndex,
+        mobility_scan_num_range_begin + (mobility_scan_num_range_width / 2));
+      mass_spectrum.setParameterValue(
+        QualifiedMassSpectrumParameter::TimsFrameIonMobScanIndexBegin,
+        mobility_scan_num_range_begin);
+      mass_spectrum.setParameterValue(
+        QualifiedMassSpectrumParameter::TimsFrameIonMobScanIndexEnd,
+        mobility_scan_num_range_end);
 
       mass_spectrum.setParameterValue(
         QualifiedMassSpectrumParameter::TimsFrameScansCount,
-        (qlonglong)tims_frame_csp->getTotalNumberOfScans());
-      mass_spectrum.setParameterValue(
-        QualifiedMassSpectrumParameter::TimsFrameInvKoBegin,
-        tims_frame_csp->getOneOverK0Transformation(0));
-      mass_spectrum.setParameterValue(
-        QualifiedMassSpectrumParameter::TimsFrameInvKoEnd,
-        tims_frame_csp->getOneOverK0Transformation(
-          tims_frame_csp->getTotalNumberOfScans() - 1));
+        static_cast<qlonglong>(tims_frame_csp->getTotalNumberOfScans()));
 
-      // qDebug() << "Setting mass spectrum parameter value mz index begin to:"
-      //          << min_mz_index_out;
-      mass_spectrum.setParameterValue(
-        QualifiedMassSpectrumParameter::TimsFrameMzIndexBegin,
-        min_mz_index_out);
-
-      // qDebug() << "Setting mass spectrum parameter value mz index end to:"
-      //          << max_mz_index_out;
-      mass_spectrum.setParameterValue(
-        QualifiedMassSpectrumParameter::TimsFrameMzIndexEnd, max_mz_index_out);
+      Trace trace;
 
       if(config.needPeakList())
         {
-          mass_spectrum.setEmptyMassSpectrum(false);
+          // Provide these two variables for the function below to fill in the
+          // values. We will need them later.
+          quint32 min_mz_index_out = 0;
+          quint32 max_mz_index_out = 0;
+
+          if(asked_mz_range)
+            {
+              trace = tims_frame_csp->cumulateScansToTraceMzDownResolution2(
+                mz_index_merge_window,
+                mz_range_begin,
+                mz_range_end,
+                mobility_scan_num_range_begin,
+                mobility_scan_num_range_end,
+                min_mz_index_out,
+                max_mz_index_out);
+            }
+          else
+            {
+              trace = tims_frame_csp->cumulateScansToTraceMzDownResolution(
+                mz_index_merge_window,
+                mobility_scan_num_range_begin,
+                mobility_scan_num_range_end,
+                min_mz_index_out,
+                max_mz_index_out);
+            }
+
+          // qDebug() << "Got min_mz_index_out:" << min_mz_index_out;
+          // qDebug() << "Got max_mz_index_out:" << max_mz_index_out;
+
+          mass_spectrum.setParameterValue(
+            QualifiedMassSpectrumParameter::TimsFrameMzIndexBegin,
+            min_mz_index_out);
+          mass_spectrum.setParameterValue(
+            QualifiedMassSpectrumParameter::TimsFrameMzIndexEnd,
+            max_mz_index_out);
 
           mass_spectrum.setMassSpectrumSPtr(
-            std::make_shared<MassSpectrum>(spectrum));
+            std::make_shared<MassSpectrum>(trace));
+          mass_spectrum.setEmptyMassSpectrum(false);
         }
       else
         {
