@@ -307,43 +307,44 @@ MsRunReaderTicChromatogram::getTicChromatogram() const
   return m_ticChromMapTrace.toTrace();
 }
 
-MsRunReaderQualifiedSpectrumList::MsRunReaderQualifiedSpectrumList()
+MsRunQualifiedSpectrumLoader::MsRunQualifiedSpectrumLoader()
 {
 }
 
 
-MsRunReaderQualifiedSpectrumList::~MsRunReaderQualifiedSpectrumList()
+MsRunQualifiedSpectrumLoader::~MsRunQualifiedSpectrumLoader()
 {
 }
 
 
 bool
-MsRunReaderQualifiedSpectrumList::needPeakList() const
+MsRunQualifiedSpectrumLoader::needPeakList() const
 {
   return true;
 }
 
 void
-MsRunReaderQualifiedSpectrumList::spectrumListHasSize(std::size_t size)
+MsRunQualifiedSpectrumLoader::spectrumListHasSize(std::size_t size)
 {
   m_qualifiedSpectrumList.clear();
 }
 
 void
-MsRunReaderQualifiedSpectrumList::setQualifiedMassSpectrum(
+MsRunQualifiedSpectrumLoader::setQualifiedMassSpectrum(
   const QualifiedMassSpectrum &qualified_mass_spectrum)
 {
+  qDebug() << qualified_mass_spectrum.getMassSpectrumId().getNativeId();
   m_qualifiedSpectrumList.push_back(qualified_mass_spectrum);
 }
 
 const std::vector<QualifiedMassSpectrum> &
-MsRunReaderQualifiedSpectrumList::getQualifiedMassSpectrumList() const
+MsRunQualifiedSpectrumLoader::getQualifiedMassSpectrumList() const
 {
   return m_qualifiedSpectrumList;
 }
 
 void
-MsRunReaderQualifiedSpectrumList::clear()
+MsRunQualifiedSpectrumLoader::clear()
 {
   m_qualifiedSpectrumList.clear();
 }
