@@ -116,7 +116,7 @@ enum class Isotope
 /** \def MzFormat mz data file format types
  *
  */
-enum class MzFormat : std::int8_t
+enum class MsDataFormat : std::int8_t
 {
   unknown           = 0, ///< unknown format
   mzML              = 1, ///< mzML
@@ -135,7 +135,8 @@ enum class MzFormat : std::int8_t
   brukerYep         = 14,
   brukerBaf         = 15,
   brukerTims        = 16,
-  last              = 17
+  brukerBafAscii    = 17, // Baf to ascii from Bruker software
+  last              = 18
 };
 
 
@@ -144,11 +145,14 @@ enum class MzFormat : std::int8_t
  */
 enum class FileReaderType
 {
-  pwiz,
+  pwiz, ///< using libpwizlite
   xy,
-  tims,
-  tims_frames,
-  tims_ms2,
+  bafascii,
+  tims,        ///< TimsMsRunReader : each scan is returned as a mass spectrum
+  tims_frames, ///< TimsFramesMsRunReader : the whole frame is merged in a
+               ///< single spectrum
+  tims_ms2, ///< TimsMsRunReaderMs2 : Spectrum are delivered for each precursor
+            ///< (MS1 or MS2)
 };
 
 
