@@ -386,7 +386,7 @@ TimsData::getRawIndexFromCoordinate(std::size_t frame_id,
     }
 
   throw ExceptionNotFound(
-    QObject::tr("ERROR raw index with frame=%1 scan=%2 not found")
+    QObject::tr("ERROR raw index with frame_id=%1 scan_index=%2 not found")
       .arg(frame_id)
       .arg(scan_num));
 }
@@ -803,10 +803,11 @@ TimsData::getQualifiedMassSpectrumByRawIndex(
 
       spectrum_id.setSpectrumIndex(spectrum_index);
       spectrum_id.setMsRunId(msrun_id);
-      spectrum_id.setNativeId(QString("frame=%1 scan=%2 index=%3")
-                                .arg(coordinate.first)
-                                .arg(coordinate.second)
-                                .arg(spectrum_index));
+      spectrum_id.setNativeId(
+        QString("frame_id=%1 scan_index=%2 global_scan_index=%3")
+          .arg(coordinate.first)
+          .arg(coordinate.second)
+          .arg(spectrum_index));
 
       mass_spectrum.setMassSpectrumId(spectrum_id);
 
@@ -817,7 +818,7 @@ TimsData::getQualifiedMassSpectrumByRawIndex(
         tims_frame.get()->getDriftTime(coordinate.second));
       // 1/K0
       mass_spectrum.setParameterValue(
-        QualifiedMassSpectrumParameter::OneOverK0,
+        QualifiedMassSpectrumParameter::TimsIonMobScanOneOverK0,
         tims_frame.get()->getOneOverK0Transformation(coordinate.second));
 
       mass_spectrum.setEmptyMassSpectrum(true);
@@ -854,7 +855,7 @@ TimsData::getQualifiedMassSpectrumByRawIndex(
 
               mass_spectrum.setPrecursorSpectrumIndex(prec_spectrum_index);
               mass_spectrum.setPrecursorNativeId(
-                QString("frame=%1 scan=%2 index=%3")
+                QString("frame_id=%1 scan_index=%2 global_scan_index=%3")
                   .arg(spectrum_descr.parent_frame)
                   .arg(coordinate.second)
                   .arg(prec_spectrum_index));
@@ -863,7 +864,7 @@ TimsData::getQualifiedMassSpectrumByRawIndex(
                 QualifiedMassSpectrumParameter::IsolationMz,
                 spectrum_descr.isolationMz);
               mass_spectrum.setParameterValue(
-                QualifiedMassSpectrumParameter::IsolationWidth,
+                QualifiedMassSpectrumParameter::IsolationMzWidth,
                 spectrum_descr.isolationWidth);
 
               mass_spectrum.setParameterValue(
@@ -993,7 +994,7 @@ TimsData::getQualifiedMs1MassSpectrumByPrecursorId(
       MassSpectrumId spectrum_id;
       spectrum_id.setSpectrumIndex(spectrum_descr.ms1_index);
       spectrum_id.setNativeId(
-        QString("frame=%1 begin=%2 end=%3 precursor=%4 idxms1=%5")
+        QString("frame_id=%1 begin=%2 end=%3 precursor=%4 idxms1=%5")
           .arg(spectrum_descr.parent_frame)
           .arg(spectrum_descr.scan_mobility_start)
           .arg(spectrum_descr.scan_mobility_end)
@@ -1019,12 +1020,12 @@ TimsData::getQualifiedMs1MassSpectrumByPrecursorId(
       mass_spectrum.setRtInSeconds(tims_frame.get()->getTime());
 
       mass_spectrum.setParameterValue(
-        QualifiedMassSpectrumParameter::OneOverK0begin,
+        QualifiedMassSpectrumParameter::TimsFrameIonMobOneOverK0Begin,
         tims_frame.get()->getOneOverK0Transformation(
           spectrum_descr.scan_mobility_start));
 
       mass_spectrum.setParameterValue(
-        QualifiedMassSpectrumParameter::OneOverK0end,
+        QualifiedMassSpectrumParameter::TimsFrameIonMobOneOverK0End,
         tims_frame.get()->getOneOverK0Transformation(
           spectrum_descr.scan_mobility_end));
 
@@ -1286,11 +1287,12 @@ TimsData::getXicCoordTimsFromPrecursorId(std::size_t precursor_id,
 }
 
 
-std::map<quint32, quint32>
+TimsDataFastMap &
 TimsData::getRawMs2ByPrecursorId(std::size_t precursor_index)
 {
   qDebug();
-  std::map<quint32, quint32> raw_spectrum;
+  TimsDataFastMap &raw_spectrum = TimsDataFastMap::getTimsDataFastMapInstance();
+  raw_spectrum.tofIndexList.clear();
   try
     {
       QSqlDatabase qdb = openDatabaseConnection();
@@ -1448,7 +1450,7 @@ TimsData::getQualifiedMs2MassSpectrumByPrecursorId(
       mass_spectrum.appendPrecursorIonData(spectrum_descr.precursor_ion_data);
 
       mass_spectrum.setPrecursorNativeId(
-        QString("frame=%1 begin=%2 end=%3 precursor=%4 idxms1=%5")
+        QString("frame_id=%1 begin=%2 end=%3 precursor=%4 idxms1=%5")
           .arg(spectrum_descr.parent_frame)
           .arg(spectrum_descr.scan_mobility_start)
           .arg(spectrum_descr.scan_mobility_end)
@@ -1459,7 +1461,7 @@ TimsData::getQualifiedMs2MassSpectrumByPrecursorId(
         QualifiedMassSpectrumParameter::IsolationMz,
         spectrum_descr.isolationMz);
       mass_spectrum.setParameterValue(
-        QualifiedMassSpectrumParameter::IsolationWidth,
+        QualifiedMassSpectrumParameter::IsolationMzWidth,
         spectrum_descr.isolationWidth);
 
       mass_spectrum.setParameterValue(
@@ -1474,7 +1476,9 @@ TimsData::getQualifiedMs2MassSpectrumByPrecursorId(
       TimsFrameBaseCstSPtr tims_frame, previous_frame;
       // TracePlusCombiner combiner;
       // MapTrace combiner_result;
-      std::map<quint32, quint32> raw_spectrum;
+      TimsDataFastMap &raw_spectrum =
+        TimsDataFastMap::getTimsDataFastMapInstance();
+      raw_spectrum.tofIndexList.clear();
       bool first = true;
       for(std::size_t tims_id : spectrum_descr.tims_frame_list)
         {
@@ -1498,12 +1502,12 @@ TimsData::getQualifiedMs2MassSpectrumByPrecursorId(
               mass_spectrum.setRtInSeconds(tims_frame.get()->getTime());
 
               mass_spectrum.setParameterValue(
-                QualifiedMassSpectrumParameter::OneOverK0begin,
+                QualifiedMassSpectrumParameter::TimsFrameIonMobOneOverK0Begin,
                 tims_frame.get()->getOneOverK0Transformation(
                   spectrum_descr.scan_mobility_start));
 
               mass_spectrum.setParameterValue(
-                QualifiedMassSpectrumParameter::OneOverK0end,
+                QualifiedMassSpectrumParameter::TimsFrameIonMobOneOverK0End,
                 tims_frame.get()->getOneOverK0Transformation(
                   spectrum_descr.scan_mobility_end));
 
@@ -1564,17 +1568,11 @@ TimsData::getQualifiedMs2MassSpectrumByPrecursorId(
           Trace trace;
           if(m_builtinMs2Centroid)
             {
-              trace =
-                tims_frame.get()->getTraceFromCumulatedScansBuiltinCentroid(
-                  raw_spectrum);
+              //raw_spectrum.removeArtefactualSpike();
+              raw_spectrum.builtInCentroid();
             }
-          else
-            {
-              // no builtin centroid:
 
-              trace =
-                tims_frame.get()->getTraceFromCumulatedScans(raw_spectrum);
-            }
+          trace = tims_frame.get()->getTraceFromCumulatedScans(raw_spectrum);
 
           if(trace.size() > 0)
             {
@@ -2255,10 +2253,11 @@ TimsData::rawReaderSpectrumCollectionByMsLevel(
 
               spectrum_id.setSpectrumIndex(i);
               spectrum_id.setMsRunId(msrun_id);
-              spectrum_id.setNativeId(QString("frame=%1 scan=%2 index=%3")
-                                        .arg(current_frame.m_frameId)
-                                        .arg(scan_num)
-                                        .arg(i));
+              spectrum_id.setNativeId(
+                QString("frame_id=%1 scan_index=%2 global_scan_index=%3")
+                  .arg(current_frame.m_frameId)
+                  .arg(scan_num)
+                  .arg(i));
 
               mass_spectrum.setMassSpectrumId(spectrum_id);
 
@@ -2269,7 +2268,7 @@ TimsData::rawReaderSpectrumCollectionByMsLevel(
                 tims_frame.get()->getDriftTime(scan_num));
               // 1/K0
               mass_spectrum.setParameterValue(
-                QualifiedMassSpectrumParameter::OneOverK0,
+                QualifiedMassSpectrumParameter::TimsIonMobScanOneOverK0,
                 tims_frame.get()->getOneOverK0Transformation(scan_num));
 
               mass_spectrum.setEmptyMassSpectrum(true);
@@ -2317,7 +2316,8 @@ TimsData::rawReaderSpectrumCollectionByMsLevel(
                       mass_spectrum.setPrecursorSpectrumIndex(
                         prec_spectrum_index);
                       mass_spectrum.setPrecursorNativeId(
-                        QString("frame=%1 scan=%2 index=%3")
+                        QString(
+                          "frame_id=%1 scan_index=%2 global_scan_index=%3")
                           .arg(spectrum_descr.parent_frame)
                           .arg(scan_num)
                           .arg(prec_spectrum_index));
@@ -2326,7 +2326,7 @@ TimsData::rawReaderSpectrumCollectionByMsLevel(
                         QualifiedMassSpectrumParameter::IsolationMz,
                         spectrum_descr.isolationMz);
                       mass_spectrum.setParameterValue(
-                        QualifiedMassSpectrumParameter::IsolationWidth,
+                        QualifiedMassSpectrumParameter::IsolationMzWidth,
                         spectrum_descr.isolationWidth);
 
                       mass_spectrum.setParameterValue(
@@ -2345,7 +2345,7 @@ TimsData::rawReaderSpectrumCollectionByMsLevel(
     }
 }
 
-std::map<quint32, quint32>
+TimsDataFastMap &
 TimsData::getRawMsBySpectrumIndex(std::size_t spectrum_index)
 {
 
@@ -2354,7 +2354,8 @@ TimsData::getRawMsBySpectrumIndex(std::size_t spectrum_index)
   TimsFrameBaseCstSPtr tims_frame;
   tims_frame = getTimsFrameCstSPtrCached(coordinate.first);
 
-  std::map<quint32, quint32> raw_spectrum;
+  TimsDataFastMap &raw_spectrum = TimsDataFastMap::getTimsDataFastMapInstance();
+  raw_spectrum.tofIndexList.clear();
   tims_frame.get()->cumulateScansInRawMap(
     raw_spectrum, coordinate.second, coordinate.second);
   return raw_spectrum;
