@@ -32,7 +32,6 @@
 #include <QtEndian>
 #include <memory>
 
-
 namespace pappso
 {
 
@@ -303,7 +302,7 @@ TimsFrame::cumulateScansIntensities(std::size_t mobility_scan_begin,
 
 void
 TimsFrame::cumulateScan(std::size_t scanNum,
-                        std::map<quint32, quint32> &accumulate_into) const
+                        TimsDataFastMap &accumulate_into) const
 {
   // qDebug();
 
@@ -327,13 +326,7 @@ TimsFrame::cumulateScan(std::size_t scanNum,
 
       previous = x;
 
-      auto ret = accumulate_into.insert(std::pair<quint32, quint32>(x, y));
-
-      if(ret.second == false)
-        {
-          // already existed : cumulate
-          ret.first->second += y;
-        }
+      accumulate_into.accumulateIntensity(x, y);
     }
 
   // qDebug();
@@ -342,9 +335,7 @@ TimsFrame::cumulateScan(std::size_t scanNum,
 // Proof of concept m/z range-conditions for accumulations
 void
 TimsFrame::cumulateScan2(std::size_t scanNum,
-                         std::map<quint32, quint32> &accumulate_into,
-                         double mz_range_begin,
-                         double mz_range_end,
+                         TimsDataFastMap &accumulate_into,
                          quint32 accepted_tof_index_range_begin,
                          quint32 accepted_tof_index_range_end) const
 {
@@ -383,13 +374,7 @@ TimsFrame::cumulateScan2(std::size_t scanNum,
 
       previous = x;
 
-      auto ret = accumulate_into.insert(std::pair<quint32, quint32>(x, y));
-
-      if(ret.second == false)
-        {
-          // already existed : cumulate
-          ret.first->second += y;
-        }
+      accumulate_into.accumulateIntensity(x, y);
     }
 
   // qDebug();
@@ -398,7 +383,7 @@ TimsFrame::cumulateScan2(std::size_t scanNum,
 
 Trace
 TimsFrame::cumulateScansToTrace(std::size_t mobility_scan_begin,
-                               std::size_t mobility_scan_end) const
+                                std::size_t mobility_scan_end) const
 {
   // qDebug();
 
@@ -408,7 +393,9 @@ TimsFrame::cumulateScansToTrace(std::size_t mobility_scan_begin,
     {
       if(m_timsDataFrame.size() == 0)
         return new_trace;
-      std::map<quint32, quint32> raw_spectrum;
+      TimsDataFastMap &raw_spectrum =
+        TimsDataFastMap::getTimsDataFastMapInstance();
+      raw_spectrum.tofIndexList.clear();
       // double local_accumulationTime = 0;
 
       std::size_t mobility_scan_max = mobility_scan_end + 1;
@@ -431,13 +418,12 @@ TimsFrame::cumulateScansToTrace(std::size_t mobility_scan_begin,
         getMzCalibrationInterfaceSPtr().get();
 
 
-      for(std::pair<quint32, quint32> pair_tof_intensity : raw_spectrum)
+      for(quint32 tof_index : raw_spectrum.tofIndexList)
         {
-          data_point_cumul.x =
-            mz_calibration_p->getMzFromTofIndex(pair_tof_intensity.first);
+          data_point_cumul.x = mz_calibration_p->getMzFromTofIndex(tof_index);
           // normalization
-          data_point_cumul.y =
-            pair_tof_intensity.second * ((double)100.0 / m_accumulationTime);
+          data_point_cumul.y = raw_spectrum.readIntensity(tof_index) *
+                               ((double)100.0 / m_accumulationTime);
           new_trace.push_back(data_point_cumul);
         }
       new_trace.sortX();
@@ -478,7 +464,9 @@ TimsFrame::cumulateScansToTraceMzDownResolution(
       // Allocate a map for (TOF,intensity) pairs to
       // accumulate ion mobility scans.
 
-      std::map<quint32, quint32> raw_spectrum;
+      TimsDataFastMap &raw_spectrum =
+        TimsDataFastMap::getTimsDataFastMapInstance();
+      raw_spectrum.tofIndexList.clear();
       // double local_accumulationTime = 0;
 
       std::size_t mobility_scan_max = mobility_scan_end + 1;
@@ -504,27 +492,28 @@ TimsFrame::cumulateScansToTraceMzDownResolution(
       // If the caller asks that m/z values be binned larger than they are,
       // ask that the m/z raw map be reduced in resolution.
       if(mz_index_merge_window > 0)
-        raw_spectrum = downsizeMzRawMap(mz_index_merge_window, raw_spectrum);
+        {
+          raw_spectrum.downsizeMzRawMap(mz_index_merge_window);
+        }
 
       // Store the first mz index and the last mz index of the current spectrum.
       // The values are set to the out parameters.
       mz_minimum_index_out = std::numeric_limits<quint32>::max();
       mz_maximum_index_out = 0;
 
-      for(std::pair<quint32, quint32> pair_tof_intensity : raw_spectrum)
+      for(quint32 tof_index : raw_spectrum.tofIndexList)
         {
-          if(pair_tof_intensity.first > mz_maximum_index_out)
-            mz_maximum_index_out = pair_tof_intensity.first;
-          if(pair_tof_intensity.first < mz_minimum_index_out)
-            mz_minimum_index_out = pair_tof_intensity.first;
+          if(tof_index > mz_maximum_index_out)
+            mz_maximum_index_out = tof_index;
+          if(tof_index < mz_minimum_index_out)
+            mz_minimum_index_out = tof_index;
 
           // Convert the TOF index to m/z
-          data_point_cumul.x =
-            mz_calibration_p->getMzFromTofIndex(pair_tof_intensity.first);
+          data_point_cumul.x = mz_calibration_p->getMzFromTofIndex(tof_index);
 
           // Normalization
-          data_point_cumul.y =
-            pair_tof_intensity.second * ((double)100.0 / m_accumulationTime);
+          data_point_cumul.y = raw_spectrum.readIntensity(tof_index) *
+                               ((double)100.0 / m_accumulationTime);
 
           // Finally make the data point a new Trace point.
           new_trace.push_back(data_point_cumul);
@@ -564,7 +553,8 @@ TimsFrame::cumulateScansToTraceMzDownResolution2(
   quint32 &mz_minimum_index_out,
   quint32 &mz_maximum_index_out) const
 {
-  // qDebug();
+  qDebug() << "Calling cumulateScansToTraceMzDownResolution2 for both mz and "
+              "im ranges accounting.";
 
   Trace new_trace;
 
@@ -579,32 +569,33 @@ TimsFrame::cumulateScansToTraceMzDownResolution2(
       // Allocate a map for (TOF index,intensity) pairs to
       // accumulate ion mobility scans.
 
-      std::map<quint32, quint32> raw_spectrum;
+      TimsDataFastMap &raw_spectrum =
+        TimsDataFastMap::getTimsDataFastMapInstance();
+      raw_spectrum.tofIndexList.clear();
       // double local_accumulationTime = 0;
 
       std::size_t mobility_scan_max = mobility_scan_end + 1;
 
-      quint32 only_in_mz_begin_tof_index_range =
+      quint32 tof_index_for_mz_range_begin =
         getMzCalibrationInterfaceSPtr().get()->getTofIndexFromMz(
           mz_range_begin);
-      quint32 only_in_mz_end_tof_index_range =
+      quint32 tof_index_for_mz_range_end =
         getMzCalibrationInterfaceSPtr().get()->getTofIndexFromMz(mz_range_end);
 
-      // qDebug() << "20231129 TOF index range begin:"
-      //          << only_in_mz_begin_tof_index_range;
-      // qDebug() << "20231129 TOF index range end:"
-      //          << only_in_mz_end_tof_index_range;
+      qDebug() << "20240529 TOF index for mz range begin:"
+               << tof_index_for_mz_range_begin;
+      qDebug() << "20240529 TOF index for mz range end:"
+               << tof_index_for_mz_range_end;
 
-      for(std::size_t i = mobility_scan_begin; i < mobility_scan_max; i++)
+      for(std::size_t iter = mobility_scan_begin; iter < mobility_scan_max;
+          iter++)
         {
           // qDebug() << "Going to cumulate currently iterated mobility scan:"
-          // << i;
-          cumulateScan2(i,
+          // << iter;
+          cumulateScan2(iter,
                         raw_spectrum,
-                        mz_range_begin,
-                        mz_range_end,
-                        only_in_mz_begin_tof_index_range,
-                        only_in_mz_end_tof_index_range);
+                        tof_index_for_mz_range_begin,
+                        tof_index_for_mz_range_end);
           // qDebug() << "Done cumulating currently iterated mobility scan:" <<
           // i;
 
@@ -621,27 +612,29 @@ TimsFrame::cumulateScansToTraceMzDownResolution2(
       // If the caller asks that m/z values be binned larger than they are,
       // ask that the m/z raw map be reduced in resolution.
       if(mz_index_merge_window > 0)
-        raw_spectrum = downsizeMzRawMap(mz_index_merge_window, raw_spectrum);
+        {
+          raw_spectrum.downsizeMzRawMap(mz_index_merge_window);
+        }
 
       // Store the first mz index and the last mz index of the current spectrum.
       // The values are set to the out parameters.
       mz_minimum_index_out = std::numeric_limits<quint32>::max();
       mz_maximum_index_out = 0;
 
-      for(std::pair<quint32, quint32> pair_tof_intensity : raw_spectrum)
+      // for(std::pair<quint32, quint32> pair_tof_intensity : raw_spectrum)
+      for(quint32 tof_index : raw_spectrum.tofIndexList)
         {
-          if(pair_tof_intensity.first > mz_maximum_index_out)
-            mz_maximum_index_out = pair_tof_intensity.first;
-          if(pair_tof_intensity.first < mz_minimum_index_out)
-            mz_minimum_index_out = pair_tof_intensity.first;
+          std::size_t intensity = raw_spectrum.readIntensity(tof_index);
+          if(tof_index > mz_maximum_index_out)
+            mz_maximum_index_out = tof_index;
+          if(tof_index < mz_minimum_index_out)
+            mz_minimum_index_out = tof_index;
 
           // Convert the TOF index to m/z
-          data_point_cumul.x =
-            mz_calibration_p->getMzFromTofIndex(pair_tof_intensity.first);
+          data_point_cumul.x = mz_calibration_p->getMzFromTofIndex(tof_index);
 
           // Normalization
-          data_point_cumul.y =
-            pair_tof_intensity.second * ((double)100.0 / m_accumulationTime);
+          data_point_cumul.y = intensity * ((double)100.0 / m_accumulationTime);
 
           // Finally make the data point a new Trace point.
           new_trace.push_back(data_point_cumul);
@@ -672,7 +665,7 @@ TimsFrame::cumulateScansToTraceMzDownResolution2(
 
 
 void
-TimsFrame::cumulateScansInRawMap(std::map<quint32, quint32> &rawSpectrum,
+TimsFrame::cumulateScansInRawMap(TimsDataFastMap &rawSpectrum,
                                  std::size_t mobility_scan_begin,
                                  std::size_t mobility_scan_end) const
 {
@@ -708,14 +701,6 @@ TimsFrame::cumulateScansInRawMap(std::map<quint32, quint32> &rawSpectrum,
 }
 
 
-pappso::MassSpectrumCstSPtr
-TimsFrame::getMassSpectrumCstSPtr(std::size_t scanNum) const
-{
-  // qDebug();
-
-  return getMassSpectrumSPtr(scanNum);
-}
-
 pappso::MassSpectrumSPtr
 TimsFrame::getMassSpectrumSPtr(std::size_t scanNum) const
 {
@@ -772,6 +757,67 @@ TimsFrame::getMassSpectrumSPtr(std::size_t scanNum) const
 }
 
 
+Trace
+TimsFrame::getMobilityScan(std::size_t scanNum,
+                           std::size_t mz_index_merge_window,
+                           double mz_range_begin,
+                           double mz_range_end,
+                           quint32 &mz_minimum_index_out,
+                           quint32 &mz_maximum_index_out) const
+{
+  qDebug() << "mz_range_begin:" << mz_range_begin
+           << "mz_range_end:" << mz_range_end
+           << "mz_index_merge_window:" << mz_index_merge_window;
+
+  Trace spectrum;
+
+  quint32 mz_index_begin = 0;
+  quint32 mz_index_end   = std::numeric_limits<quint32>::max();
+
+  if(mz_range_end > 0)
+    {
+      qDebug() << "m/z range is requested.";
+
+      mz_index_begin =
+        msp_mzCalibration.get()->getTofIndexFromMz(mz_range_begin);
+      mz_index_end = msp_mzCalibration.get()->getTofIndexFromMz(mz_range_end);
+    }
+
+  qDebug() << "After conversion of mz indices, mz_index_begin:"
+           << mz_index_begin << "mz_index_end;" << mz_index_end;
+
+  auto raw_spectrum =
+    getRawValuePairList(scanNum, mz_index_begin, mz_index_end);
+
+  qDebug() << " raw_spectrum.size();" << raw_spectrum.size();
+
+  if(mz_index_merge_window > 0)
+    {
+      qDebug() << "mz_index_merge_window;=" << mz_index_merge_window
+               << " raw_spectrum.size()=" << raw_spectrum.size();
+      raw_spectrum =
+        downsizeMzRawValuePairList(mz_index_merge_window, raw_spectrum);
+    }
+
+  if(raw_spectrum.size() > 0)
+    {
+      mz_minimum_index_out = raw_spectrum.front().mz_tof_index;
+      mz_maximum_index_out = raw_spectrum.back().mz_tof_index;
+
+      for(auto &&element : raw_spectrum)
+        {
+          spectrum.push_back(DataPoint(
+            msp_mzCalibration.get()->getMzFromTofIndex(element.mz_tof_index),
+            static_cast<double>(element.intensity_index)));
+
+          // intensity normalization
+          spectrum.back().y *= 100.0 / m_accumulationTime;
+        }
+    }
+
+  return spectrum;
+}
+
 void
 TimsFrame::extractTimsXicListInRtRange(
   std::vector<XicCoordTims *>::iterator &itXicListbegin,
@@ -902,4 +948,54 @@ TimsFrame::getRawTraceSPtr(std::size_t scanNum) const
   return trace_sptr;
 }
 
+
+std::vector<TimsFrame::RawValuePair>
+TimsFrame::getRawValuePairList(std::size_t scanNum,
+                               quint32 accepted_tof_index_range_begin,
+                               quint32 accepted_tof_index_range_end) const
+{
+  // qDebug() << accepted_tof_index_range_begin;
+
+  std::vector<TimsFrame::RawValuePair> raw_value_pairs;
+  // std::vector<DataPoint>
+
+  if(m_timsDataFrame.size() == 0)
+    return raw_value_pairs;
+  // qDebug();
+
+  std::size_t size = getNbrPeaks(scanNum);
+
+  std::size_t offset = getScanOffset(scanNum);
+
+  // qDebug() << size;
+  qint32 previous = -1;
+  std::vector<quint32> index_list;
+  for(std::size_t i = 0; i < size; i++)
+    {
+
+      // qDebug() << i;
+      TimsFrame::RawValuePair raw_value_pair(
+        {(*(quint32 *)((m_timsDataFrame.constData() + (offset * 4) + (i * 8))) +
+          previous),
+         (*(quint32 *)(m_timsDataFrame.constData() + (offset * 4) + (i * 8) +
+                       4))});
+
+      previous = raw_value_pair.mz_tof_index;
+      if(raw_value_pair.mz_tof_index < accepted_tof_index_range_begin)
+        {
+          // qDebug() << "TOF index still not in range, x:" << x;
+          continue;
+        }
+      if(raw_value_pair.mz_tof_index > accepted_tof_index_range_end)
+        {
+          // qDebug() << "TOF index already out of range, x:" << x;
+          break;
+        }
+
+      raw_value_pairs.push_back(raw_value_pair);
+    }
+  qDebug() << raw_value_pairs.size();
+  return raw_value_pairs;
+}
+
 } // namespace pappso
