@@ -34,6 +34,7 @@
 #include "../../xic/xic.h"
 #include "../../msrun/xiccoord/xiccoordtims.h"
 
+
 namespace pappso
 {
 
@@ -84,7 +85,7 @@ class TimsFrame : public TimsFrameBase
    * @return Trace mz and intensity values
    */
   virtual Trace cumulateScansToTrace(std::size_t scanNumBegin,
-                                    std::size_t scanNumEnd) const override;
+                                     std::size_t scanNumEnd) const override;
 
 
   /** @brief cumulate spectrum given a scan number range
@@ -99,32 +100,80 @@ class TimsFrame : public TimsFrameBase
    * file, from 0 to N-1
    * @param scanNumEnd scan number in the frame in the order it lies in binary
    * file, from 0 to N-1
+   * @param mz_minimum_index report the minimum mz index contained in the
+   * resulting trace
+   * @param mz_maximum_index report the maximum mz index contained in the
+   * resulting trace
+   *
    */
-  virtual Trace
-  cumulateScansToTraceMzDownResolution(std::size_t mzindex_merge_window,
-                                      std::size_t scanNumBegin,
-                                      std::size_t scanNumEnd,
-                                      quint32 &minimum_index,
-                                      quint32 &maximum_index) const override;
+  virtual Trace cumulateScansToTraceMzDownResolution(
+    std::size_t mzindex_merge_window,
+    std::size_t scanNumBegin,
+    std::size_t scanNumEnd,
+    quint32 &mz_minimum_index,
+    quint32 &mz_maximum_index) const override;
 
-  Trace
-  cumulateScansToTraceMzDownResolution2(std::size_t mz_index_merge_window,
-                                       double mz_range_begin,
-                                       double mz_range_end,
-                                       std::size_t mobility_scan_begin,
-                                       std::size_t mobility_scan_end,
-                                       quint32 &mz_minimum_index_out,
-                                       quint32 &mz_maximum_index_out) const;
+
+  /** @brief cumulate spectrum given a scan number range
+   * need the binary file
+   * The intensities are normalized with respect to the frame accumulation time
+   * to leverage computing performance, this function decreases the mz
+   * resolution
+   *
+   * @param mzindex_merge_window width of the mzindex window used to merge all
+   * intensities into a single point. This results in faster computing.
+   * @param mz_range_begin
+   * @param mz_range_end
+   * @param scanNumBegin scan number in the frame in the order it lies in binary
+   * file, from 0 to N-1
+   * @param scanNumEnd scan number in the frame in the order it lies in binary
+   * file, from 0 to N-1
+   * @param mz_minimum_index report the minimum mz index contained in the
+   * resulting trace (constrained by the mz_range_begin)
+   * @param mz_maximum_index report the maximum mz index contained in the
+   * resulting trace (constrained by the mz_range_end)
+   *
+   */
+  virtual Trace cumulateScansToTraceMzDownResolution2(
+    std::size_t mz_index_merge_window,
+    double mz_range_begin,
+    double mz_range_end,
+    std::size_t mobility_scan_begin,
+    std::size_t mobility_scan_end,
+    quint32 &mz_minimum_index_out,
+    quint32 &mz_maximum_index_out) const override;
 
   /** @brief cumulate scan list into a trace into a raw spectrum map
    * @param rawSpectrum simple map of integers to cumulate raw counts
    * @param scanNumBegin first scan to cumulate
    * @param scanNumEnd last scan to cumulate
    */
-  virtual void cumulateScansInRawMap(std::map<quint32, quint32> &rawSpectrum,
+  virtual void cumulateScansInRawMap(TimsDataFastMap &rawSpectrum,
                                      std::size_t scanNumBegin,
                                      std::size_t scanNumEnd) const override;
 
+
+  /** @brief get a single mobility scan m/z + intensities
+   *
+   * @param scanNum scan number in the frame in the order it lies in binary
+   * file, from 0 to N-1
+   * @param mzindex_merge_window width of the mzindex window used to merge all
+   * intensities into a single point. This results in faster computing.
+   * @param mz_range_begin
+   * @param mz_range_end
+   * @param mz_minimum_index report the minimum mz index contained in the
+   * resulting trace (constrained by the mz_range_begin)
+   * @param mz_maximum_index report the maximum mz index contained in the
+   * resulting trace (constrained by the mz_range_end)
+   *
+   */
+  virtual Trace getMobilityScan(std::size_t scanNum,
+                                std::size_t mz_index_merge_window,
+                                double mz_range_begin,
+                                double mz_range_end,
+                                quint32 &mz_minimum_index_out,
+                                quint32 &mz_maximum_index_out) const override;
+
   virtual quint64
   cumulateSingleScanIntensities(std::size_t scanNum) const override;
 
@@ -144,16 +193,23 @@ class TimsFrame : public TimsFrameBase
   virtual std::vector<quint32>
   getScanIntensities(std::size_t scanNum) const override;
 
-  /** @brief get the mass spectrum corresponding to a scan number
-   * @param scanNum the scan number to retrieve
-   * */
-  virtual pappso::MassSpectrumCstSPtr
-  getMassSpectrumCstSPtr(std::size_t scanNum) const;
-
   virtual pappso::MassSpectrumSPtr
   getMassSpectrumSPtr(std::size_t scanNum) const override;
 
 
+  /** @brief get the raw index tof_index and intensities (normalized)
+   *
+   * @param scanNum the scan number to extract
+   * @param accepted_tof_index_range_begin mz index begin
+   * @param accepted_tof_index_range_end mz index end
+   * @return vector of RawValuePair
+   *
+   */
+  virtual std::vector<RawValuePair>
+  getRawValuePairList(std::size_t scanNum,
+                      quint32 accepted_tof_index_range_begin,
+                      quint32 accepted_tof_index_range_end) const;
+
   protected:
   /** @brief constructor for binary independant tims frame
    * @param timsId tims frame identifier in the database
@@ -172,15 +228,13 @@ class TimsFrame : public TimsFrameBase
    * @param scanNum scan number 0 to (m_scanNumber-1)
    */
   virtual void cumulateScan(std::size_t scanNum,
-                            std::map<quint32, quint32> &accumulate_into) const;
+                            TimsDataFastMap &accumulate_into) const;
 
 
   virtual void cumulateScan2(std::size_t scanNum,
-                             std::map<quint32, quint32> &accumulate_into,
-                             double mz_range_begin,
-                             double mz_range_end,
-                         quint32 accepted_tof_index_range_begin,
-                         quint32 accepted_tof_index_range_end) const;
+                             TimsDataFastMap &accumulate_into,
+                             quint32 accepted_tof_index_range_begin,
+                             quint32 accepted_tof_index_range_end) const;
 
   /** @brief get the raw index tof_index and intensities (normalized)
    *
@@ -225,5 +279,3 @@ class TimsFrame : public TimsFrameBase
   QByteArray m_timsDataFrame;
 };
 } // namespace pappso
-
-
