@@ -136,9 +136,18 @@ TimsFrameBase::getMassSpectrumSPtr(std::size_t scanNum) const
       "ERROR unable to getMassSpectrumSPtr in TimsFrameBase for scan number %1")
       .arg(scanNum));
 }
+
+
+pappso::MassSpectrumCstSPtr
+TimsFrameBase::getMassSpectrumCstSPtr(std::size_t scanNum) const
+{
+  // qDebug();
+
+  return getMassSpectrumSPtr(scanNum);
+}
 Trace
 TimsFrameBase::cumulateScansToTrace(std::size_t scanNumBegin,
-                                   std::size_t scanNumEnd) const
+                                    std::size_t scanNumEnd) const
 {
   throw PappsoException(
     QObject::tr("ERROR unable to cumulateScanToTrace in TimsFrameBase for scan "
@@ -161,8 +170,40 @@ TimsFrameBase::cumulateScansToTraceMzDownResolution(
                           .arg(__LINE__));
 }
 
+Trace
+TimsFrameBase::cumulateScansToTraceMzDownResolution2(
+  std::size_t mz_index_merge_window [[maybe_unused]],
+  double mz_range_begin [[maybe_unused]],
+  double mz_range_end [[maybe_unused]],
+  std::size_t mobility_scan_begin [[maybe_unused]],
+  std::size_t mobility_scan_end [[maybe_unused]],
+  quint32 &mz_minimum_index_out [[maybe_unused]],
+  quint32 &mz_maximum_index_out [[maybe_unused]]) const
+{
+  throw PappsoException(QObject::tr("Non implemented function %1 %2 %3")
+                          .arg(__FILE__)
+                          .arg(__FUNCTION__)
+                          .arg(__LINE__));
+}
+
+Trace
+TimsFrameBase::getMobilityScan(std::size_t scanNum [[maybe_unused]],
+                               std::size_t mz_index_merge_window
+                               [[maybe_unused]],
+                               double mz_range_begin [[maybe_unused]],
+                               double mz_range_end [[maybe_unused]],
+                               quint32 &mz_minimum_index_out [[maybe_unused]],
+                               quint32 &mz_maximum_index_out
+                               [[maybe_unused]]) const
+{
+  throw PappsoException(QObject::tr("Non implemented function %1 %2 %3")
+                          .arg(__FILE__)
+                          .arg(__FUNCTION__)
+                          .arg(__LINE__));
+}
+
 void
-TimsFrameBase::cumulateScansInRawMap(std::map<quint32, quint32> &rawSpectrum
+TimsFrameBase::cumulateScansInRawMap(TimsDataFastMap &rawSpectrum
                                      [[maybe_unused]],
                                      std::size_t scanNumBegin,
                                      std::size_t scanNumEnd) const
@@ -331,7 +372,7 @@ TimsFrameBase::hasSameCalibrationData(const TimsFrameBase &other) const
 
 pappso::Trace
 TimsFrameBase::getTraceFromCumulatedScans(
-  std::map<quint32, quint32> &accumulated_scans) const
+  TimsDataFastMap &accumulated_scans) const
 {
   qDebug();
   // qDebug();
@@ -343,13 +384,14 @@ TimsFrameBase::getTraceFromCumulatedScans(
 
 
   DataPoint element;
-  for(auto &scan_element : accumulated_scans)
+  for(quint32 tof_index : accumulated_scans.tofIndexList)
     {
       // intensity normalization
-      element.y = ((double)scan_element.second) * 100.0 / m_accumulationTime;
+      element.y = ((double)accumulated_scans.readIntensity(tof_index)) * 100.0 /
+                  m_accumulationTime;
 
       // mz calibration
-      element.x = mz_calibration_p->getMzFromTofIndex(scan_element.first);
+      element.x = mz_calibration_p->getMzFromTofIndex(tof_index);
 
       local_trace.push_back(element);
     }
@@ -360,115 +402,6 @@ TimsFrameBase::getTraceFromCumulatedScans(
   return local_trace;
 }
 
-pappso::Trace
-TimsFrameBase::getTraceFromCumulatedScansBuiltinCentroid(
-  std::map<quint32, quint32> &accumulated_scans) const
-{
-  qDebug();
-  // qDebug();
-  // add flanking peaks
-  std::vector<quint32> keys;
-  transform(begin(accumulated_scans),
-            end(accumulated_scans),
-            back_inserter(keys),
-            [](std::map<quint32, quint32>::value_type const &pair) {
-              return pair.first;
-            });
-  std::sort(keys.begin(), keys.end());
-  pappso::DataPoint data_point_cumul;
-  data_point_cumul.x = 0;
-  data_point_cumul.y = 0;
-
-  pappso::Trace local_trace;
-
-  MzCalibrationInterface *mz_calibration_p =
-    getMzCalibrationInterfaceSPtr().get();
-
-
-  quint32 last_key = 0;
-
-  for(quint32 key : keys)
-    {
-      if(key == last_key + 1)
-        {
-          // cumulate
-          if(accumulated_scans[key] > accumulated_scans[last_key])
-            {
-              if(data_point_cumul.x == last_key)
-                {
-                  // growing peak
-                  data_point_cumul.x = key;
-                  data_point_cumul.y += accumulated_scans[key];
-                }
-              else
-                {
-                  // new peak
-                  // flush
-                  if(data_point_cumul.y > 0)
-                    {
-                      // intensity normalization
-                      data_point_cumul.y *= 100.0 / m_accumulationTime;
-
-
-                      // mz calibration
-                      data_point_cumul.x =
-                        mz_calibration_p->getMzFromTofIndex(data_point_cumul.x);
-                      local_trace.push_back(data_point_cumul);
-                    }
-
-                  // new point
-                  data_point_cumul.x = key;
-                  data_point_cumul.y = accumulated_scans[key];
-                }
-            }
-          else
-            {
-              data_point_cumul.y += accumulated_scans[key];
-            }
-        }
-      else
-        {
-          // flush
-          if(data_point_cumul.y > 0)
-            {
-              // intensity normalization
-              data_point_cumul.y *= 100.0 / m_accumulationTime;
-
-
-              // qDebug() << "raw data x=" << data_point_cumul.x;
-              // mz calibration
-              data_point_cumul.x =
-                mz_calibration_p->getMzFromTofIndex(data_point_cumul.x);
-              // qDebug() << "mz=" << data_point_cumul.x;
-              local_trace.push_back(data_point_cumul);
-            }
-
-          // new point
-          data_point_cumul.x = key;
-          data_point_cumul.y = accumulated_scans[key];
-        }
-
-      last_key = key;
-    }
-  // flush
-  if(data_point_cumul.y > 0)
-    {
-      // intensity normalization
-      data_point_cumul.y *= 100.0 / m_accumulationTime;
-
-
-      // mz calibration
-      data_point_cumul.x =
-        mz_calibration_p->getMzFromTofIndex(data_point_cumul.x);
-      local_trace.push_back(data_point_cumul);
-    }
-
-  local_trace.sortX();
-  qDebug();
-  // qDebug();
-  return local_trace;
-}
-
 const MzCalibrationInterfaceSPtr &
 TimsFrameBase::getMzCalibrationInterfaceSPtr() const
 {
@@ -602,23 +535,46 @@ TimsFrameBase::getIonMobilityTraceByMzIndexRange(
   return im_trace;
 }
 
-std::map<quint32, quint32> &
-TimsFrameBase::downsizeMzRawMap(std::size_t mzindex_merge_window,
-                                std::map<quint32, quint32> &rawSpectrum) const
-{
-  std::map<quint32, quint32> new_spectrum;
 
+std::vector<TimsFrameBase::RawValuePair> &
+TimsFrameBase::downsizeMzRawValuePairList(
+  std::size_t mzindex_merge_window,
+  std::vector<TimsFrameBase::RawValuePair> &rawSpectrum) const
+{
+
+  qDebug() << rawSpectrum.size();
+  std::vector<TimsFrameBase::RawValuePair> new_spectrum;
+
+  TimsFrameBase::RawValuePair current_point;
+  current_point.intensity_index = 0;
+  current_point.mz_tof_index    = 0;
   for(auto &pair_mz_intensity : rawSpectrum)
     {
-      quint32 mzkey = (pair_mz_intensity.first / mzindex_merge_window);
-      mzkey   = (mzkey * mzindex_merge_window) + (mzindex_merge_window / 2);
-      auto it = new_spectrum.insert({mzkey, pair_mz_intensity.second});
-      if(it.second == false)
+      quint32 mzkey = (pair_mz_intensity.mz_tof_index / mzindex_merge_window);
+      mzkey = (mzkey * mzindex_merge_window) + (mzindex_merge_window / 2);
+
+      if(current_point.mz_tof_index != mzkey)
         {
-          it.first->second += pair_mz_intensity.second;
+          if(current_point.mz_tof_index > 0)
+            {
+              new_spectrum.push_back(current_point);
+            }
+
+          current_point.intensity_index = pair_mz_intensity.intensity_index;
+          current_point.mz_tof_index    = mzkey;
+        }
+      else
+        {
+          current_point.intensity_index += pair_mz_intensity.intensity_index;
         }
     }
+
+  if(current_point.mz_tof_index > 0)
+    {
+      new_spectrum.push_back(current_point);
+    }
   rawSpectrum = new_spectrum;
+  qDebug() << rawSpectrum.size();
   return rawSpectrum;
 }
 // namespace pappso
