@@ -217,7 +217,7 @@ TimsFrameType1::getNbrPeaks(std::size_t scanNum) const
 
 void
 TimsFrameType1::cumulateScan(std::size_t scanNum,
-                             std::map<quint32, quint32> &accumulate_into) const
+                             TimsDataFastMap &accumulate_into) const
 {
   if(m_timsDataFrame.size() == 0)
     return;
@@ -246,13 +246,60 @@ TimsFrameType1::cumulateScan(std::size_t scanNum,
           quint32 x = tof_index;
           quint32 y = value;
 
-          auto ret = accumulate_into.insert(std::pair<quint32, quint32>(x, y));
+          accumulate_into.accumulateIntensity(x, y);
 
-          if(ret.second == false)
+          tof_index++;
+        }
+    }
+  qDebug() << "end";
+}
+
+
+void
+TimsFrameType1::cumulateScan2(std::size_t scanNum,
+                              TimsDataFastMap &accumulate_into,
+                              quint32 accepted_tof_index_range_begin,
+                              quint32 accepted_tof_index_range_end) const
+{
+  if(m_timsDataFrame.size() == 0)
+    return;
+  // checkScanNum(scanNum);
+
+
+  std::size_t size = m_scanSizeList[scanNum];
+
+  std::size_t offset = m_scanOffsetList[scanNum];
+
+  // qDebug() << "begin offset=" << offset << " size=" << size;
+  qint32 value     = 0;
+  qint32 tof_index = 0;
+  for(std::size_t i = 0; i < size; i++)
+    {
+      value = (*(qint32 *)(m_timsDataFrame.constData() + offset + (i * 4)));
+      // qDebug() << " i=" << i << " value=" << value;
+
+      if(value < 0)
+        {
+          tof_index += -1 * value;
+        }
+      else
+        {
+
+          quint32 x = tof_index;
+          quint32 y = value;
+
+
+          if(x < accepted_tof_index_range_begin)
             {
-              // already existed : cumulate
-              ret.first->second += y;
+              tof_index++;
+              continue;
             }
+          if(x > accepted_tof_index_range_end)
+            {
+              break;
+            }
+          accumulate_into.accumulateIntensity(x, y);
+
           tof_index++;
         }
     }
@@ -510,4 +557,71 @@ TimsFrameType1::getRawTraceSPtr(std::size_t scanNum) const
   return trace_sptr;
 }
 
+
+std::vector<TimsFrameType1::RawValuePair>
+TimsFrameType1::getRawValuePairList(std::size_t scanNum,
+                                    quint32 accepted_tof_index_range_begin,
+                                    quint32 accepted_tof_index_range_end) const
+{
+
+  // qDebug();
+
+  std::vector<TimsFrame::RawValuePair> trace_sptr;
+  // std::vector<DataPoint>
+
+  if(m_timsDataFrame.size() == 0)
+    return trace_sptr;
+  qDebug();
+
+  std::size_t size = m_scanSizeList[scanNum];
+
+  std::size_t offset = m_scanOffsetList[scanNum];
+
+  qDebug() << " offset=" << offset << " size=" << size;
+  if(size == 0)
+    return trace_sptr;
+
+  // qDebug();
+  qint32 value     = 0;
+  qint32 tof_index = 0;
+
+  // std::vector<quint32> index_list;
+  TimsFrame::RawValuePair data_point;
+  for(std::size_t i = 0; i < size; i++)
+    {
+      value = (*(qint32 *)(m_timsDataFrame.constData() + offset + (i * 4)));
+
+      if(value < 0)
+        {
+          tof_index += -1 * value;
+        }
+      else
+        {
+          data_point.intensity_index = value;
+
+          if(data_point.intensity_index < accepted_tof_index_range_begin)
+            {
+              tof_index++;
+              continue;
+            }
+          if(data_point.intensity_index > accepted_tof_index_range_end)
+            {
+              break;
+            }
+          // intensity normalization
+          data_point.intensity_index *= 100.0 / m_accumulationTime;
+
+
+          // mz calibration
+          data_point.mz_tof_index = tof_index;
+          trace_sptr.push_back(data_point);
+          tof_index++;
+        }
+    }
+
+
+  // qDebug();
+  return trace_sptr;
+}
+
 } // namespace pappso
