@@ -89,10 +89,14 @@ class TimsFrameType1 : public TimsFrame
    *
    * @param scanNum scan number 0 to (m_scanNumber-1)
    */
-  virtual void
-  cumulateScan(std::size_t scanNum,
-               std::map<quint32, quint32> &accumulate_into) const override;
+  virtual void cumulateScan(std::size_t scanNum,
+                            TimsDataFastMap &accumulate_into) const override;
 
+  virtual void
+  cumulateScan2(std::size_t scanNum,
+                TimsDataFastMap &accumulate_into,
+                quint32 accepted_tof_index_range_begin,
+                quint32 accepted_tof_index_range_end) const override;
 
   /** @brief get the raw index tof_index and intensities (normalized)
    *
@@ -102,6 +106,20 @@ class TimsFrameType1 : public TimsFrame
    */
   virtual pappso::TraceSPtr getRawTraceSPtr(std::size_t scanNum) const override;
 
+
+  /** @brief get the raw index tof_index and intensities (normalized)
+   *
+   * @param scanNum the scan number to extract
+   * @param accepted_tof_index_range_begin mz index begin
+   * @param accepted_tof_index_range_end mz index end
+   * @return vector of RawValuePair
+   *
+   */
+  virtual std::vector<RawValuePair>
+  getRawValuePairList(std::size_t scanNum,
+                      quint32 accepted_tof_index_range_begin,
+                      quint32 accepted_tof_index_range_end) const override;
+
   private:
   /** @brief copy buffer header and lzf decompress each scan for tims
    * compression type 1
