@@ -31,13 +31,9 @@
 
 // ./tests/catch2-only-tests [timsframe] -s -a
 
-
-#ifdef CATCH2_MAJOR_VERSION_2
-#include <catch2/catch.hpp>
-#elif CATCH2_MAJOR_VERSION_3
-#include <catch2/catch_all.hpp>
-using namespace Catch;
-#endif
+#include <catch2/catch_test_macros.hpp>
+#include <catch2/catch_approx.hpp>
+#include <catch2/matchers/catch_matchers_vector.hpp>
 
 #include <QDebug>
 #include <QString>
@@ -51,6 +47,7 @@ using namespace Catch;
 #include <pappsomspp/pappsoexception.h>
 #include <pappsomspp/exception/exceptionnotfound.h>
 #include <pappsomspp/processing/filters/filtersuitestring.h>
+#include <pappsomspp/processing/filters/filterresample.h>
 #include "config.h"
 
 using namespace pappso;
@@ -85,7 +82,7 @@ TEST_CASE("Test Cardano's cubic solver", "[cubicsolver]")
     REQUIRE(res.type == CardanoResultCase::positived);
     // https://www.wolframalpha.com/widgets/view.jsp?id=578d50248844454e46e24e9ed230843d
 
-    REQUIRE(res.x1 == Approx(-0.32588));
+    REQUIRE(res.x1 == Catch::Approx(-0.32588));
   }
 
 
@@ -107,7 +104,7 @@ TEST_CASE("Test Cardano's cubic solver", "[cubicsolver]")
     REQUIRE(res.type == CardanoResultCase::quadratic);
     // https://www.wolframalpha.com/widgets/view.jsp?id=578d50248844454e46e24e9ed230843d
 
-    REQUIRE(res.x1 == Approx(-0.3271626093));
+    REQUIRE(res.x1 == Catch::Approx(-0.3271626093));
   }
 
 
@@ -129,7 +126,7 @@ TEST_CASE("Test Cardano's cubic solver", "[cubicsolver]")
     REQUIRE(res.type == CardanoResultCase::line);
     // https://www.wolframalpha.com/widgets/view.jsp?id=578d50248844454e46e24e9ed230843d
 
-    REQUIRE(res.x1 == Approx(-0.3214285714));
+    REQUIRE(res.x1 == Catch::Approx(-0.3214285714));
   }
 }
 
@@ -194,11 +191,13 @@ TEST_CASE("Test tims frames", "[timsframe]")
     // 313792
     mz = frame.getMzCalibrationInterfaceSPtr().get()->getMzFromTofIndex(
       (quint32)313793);
+
+    REQUIRE(mz == Catch::Approx(1200.002299));
     qDebug() << " 313793 => mz=" << QString::number(mz, 'g', 10);
     mz = frame.getMzCalibrationInterfaceSPtr().get()->getMzFromTofIndex(
       (quint32)313792);
 
-    REQUIRE(QString::number(mz, 'g', 10) == "1200.002299");
+    REQUIRE(mz == Catch::Approx(1200.002299));
     qDebug() << " 313792 => mz=" << QString::number(mz, 'g', 10);
     mz = frame.getMzCalibrationInterfaceSPtr().get()->getMzFromTofIndex(
       (quint32)313791);
@@ -211,12 +210,16 @@ TEST_CASE("Test tims frames", "[timsframe]")
 
     REQUIRE(scan_num == 400);
     std::cout << std::endl << "..:: Test 1 OK ::.." << std::endl;
+    mz = frame.getMzCalibrationInterfaceSPtr().get()->getMzFromTofIndex(
+      (quint32)500000);
+    REQUIRE(mz == Catch::Approx(2442.0497183326));
   }
 
   SECTION("Test 2")
   {
     std::cout << std::endl
-         << "..:: second test, new calibration parameters ::.." << std::endl;
+              << "..:: second test, new calibration parameters ::.."
+              << std::endl;
 
 
     frame.setMzCalibration(25.3072302808429,
@@ -291,6 +294,12 @@ TEST_CASE("Test tims frames", "[timsframe]")
 
 
     REQUIRE_FALSE(mz < 50.0);
+
+    REQUIRE(frame.getMzCalibrationInterfaceSPtr().get()->getTofIndexFromMz(
+              1531.6385523357) == 370436);
+    REQUIRE(frame.getMzCalibrationInterfaceSPtr().get()->getTofIndexFromMz(
+              1531.6561258431) == 370439);
+
     std::cout << std::endl << "..:: Test 2 OK ::.." << std::endl;
   }
 
@@ -306,8 +315,8 @@ TEST_CASE("Test tims frames", "[timsframe]")
       "a1");
 
 
-    accessor.setPreferredFileReaderType(pappso::MzFormat::brukerTims,
-                                       pappso::FileReaderType::tims_ms2);
+    accessor.setPreferredFileReaderType(pappso::MsDataFormat::brukerTims,
+                                        pappso::FileReaderType::tims_ms2);
 
 
     pappso::MsRunReaderSPtr p_msreader =
@@ -316,19 +325,103 @@ TEST_CASE("Test tims frames", "[timsframe]")
     REQUIRE(p_msreader != nullptr);
 
     REQUIRE(accessor.getFileReaderType() == pappso::FileReaderType::tims_ms2);
-
     pappso::TimsMsRunReaderMs2 *tims2_reader =
       dynamic_cast<pappso::TimsMsRunReaderMs2 *>(p_msreader.get());
     REQUIRE(tims2_reader != nullptr);
     if(tims2_reader != nullptr)
       {
         tims2_reader->setMs2BuiltinCentroid(true);
-        tims2_reader->setMs2FilterCstSPtr(std::make_shared<FilterSuiteString>(
-          FilterSuiteString("chargeDeconvolution|0.02dalton")));
+        tims2_reader->setMs2FilterCstSPtr(
+          std::make_shared<FilterSuiteString>(""));
       }
     pappso::QualifiedMassSpectrum mass_spectrum;
     REQUIRE_NOTHROW(mass_spectrum = tims2_reader->qualifiedMassSpectrum(5001));
+    /*
+        REQUIRE(
+          mass_spectrum.getMassSpectrumCstSPtr().get()->toString().toStdString()
+       ==
+          "");
+    */
+    Trace filter_trace(*mass_spectrum.getMassSpectrumCstSPtr().get());
 
+    // REQUIRE(filter_trace.toString().toStdString() == "");
+    QFile f(
+      QString(CMAKE_SOURCE_DIR).append("/tests/data/timstof/ms2_5001.tsv"));
+    REQUIRE(f.open(QFile::ReadOnly | QFile::Text));
+    QTextStream in(&f);
+    Trace control_trace(in.readAll());
+
+    REQUIRE_THAT(filter_trace.xValues(),
+                 Catch::Matchers::Approx(control_trace.xValues()));
+    REQUIRE_THAT(filter_trace.yValues(),
+                 Catch::Matchers::Approx(control_trace.yValues()));
+
+    // 2762 => 2291
+    // 2762 => 2322
+    REQUIRE(filter_trace.size() == 2322);
+    REQUIRE(filter_trace.sumY() == Catch::Approx(289706.9697384664));
+
+    FilterResampleKeepXRange(1075, 1078).filter(filter_trace);
+
+    REQUIRE_THAT(
+      filter_trace.xValues(),
+      Catch::Matchers::Approx(std::vector<double>({1075.5218269238,
+                                                   1075.53223643,
+                                                   1075.5426460307,
+                                                   1076.0996328182,
+                                                   1076.2922693084,
+                                                   1076.5057514656,
+                                                   1076.5317873147,
+                                                   1076.5526162383,
+                                                   1076.833826252,
+                                                   1077.521383186})));
+    REQUIRE_THAT(filter_trace.yValues(),
+                 Catch::Matchers::Approx(std::vector<double>({168.423935369,
+                                                              131.4528276051,
+                                                              54.7720115021,
+                                                              15.0623031631,
+                                                              43.8176092017,
+                                                              71.2036149528,
+                                                              246.4740517596,
+                                                              83.5273175407,
+                                                              15.0623031631,
+                                                              86.2659181158})));
+
+    FilterResampleKeepXRange(1531, 1964).filter(filter_trace);
+
+    REQUIRE_THAT(filter_trace.xValues(),
+                 Catch::Matchers::Approx(std::vector<double>({})));
+    REQUIRE_THAT(filter_trace.yValues(),
+                 Catch::Matchers::Approx(std::vector<double>({})));
+
+
+    filter_trace = (*mass_spectrum.getMassSpectrumCstSPtr().get());
+    FilterResampleKeepXRange(105, 188).filter(filter_trace);
+
+    REQUIRE_THAT(
+      filter_trace.xValues(),
+      Catch::Matchers::Approx(std::vector<double>({105.2508651592,
+                                                   122.7051378682,
+                                                   133.6175290647,
+                                                   172.8936930658,
+                                                   183.2788453809,
+                                                   186.0653089587,
+                                                   186.7109907881,
+                                                   187.021228504,
+                                                   187.1406192812,
+                                                   187.1449614799})));
+    REQUIRE_THAT(
+      filter_trace.yValues(),
+      Catch::Matchers::Approx(std::vector<double>({15.0623031631,
+                                                   15.0623031631,
+                                                   13.6930028755,
+                                                   36.9711077639,
+                                                   13.6930028755,
+                                                   15.0623031631,
+                                                   31.4939066137,
+                                                   71.2036149528,
+                                                   119.1291250171,
+                                                   105.4361221416})));
 
     /*
      * 23:   ..:: Test TIMS frame init begin ::..
@@ -346,6 +439,14 @@ TEST_CASE("Test tims frames", "[timsframe]")
     REQUIRE_THROWS_AS(mass_spectrum =
                         tims2_reader->qualifiedMassSpectrum(35460),
                       pappso::ExceptionNotFound);
+
+
+    REQUIRE_NOTHROW(mass_spectrum = tims2_reader->qualifiedMassSpectrum(5001));
+    filter_trace = *mass_spectrum.getMassSpectrumCstSPtr().get();
+    REQUIRE_THAT(filter_trace.xValues(),
+                 Catch::Matchers::Approx(control_trace.xValues()));
+    REQUIRE_THAT(filter_trace.yValues(),
+                 Catch::Matchers::Approx(control_trace.yValues()));
   }
 
 
@@ -379,8 +480,8 @@ TEST_CASE("Test tims frames", "[timsframe]")
        "a1");
     */
 
-    accessor.setPreferredFileReaderType(pappso::MzFormat::brukerTims,
-                                       pappso::FileReaderType::tims);
+    accessor.setPreferredFileReaderType(pappso::MsDataFormat::brukerTims,
+                                        pappso::FileReaderType::tims);
 
 
     pappso::MsRunReaderSPtr p_msreader_tims_ms1 =
@@ -419,8 +520,8 @@ TEST_CASE("Test tims frames", "[timsframe]")
     REQUIRE_NOTHROW(p_msreader_tims_ms1.get()->massSpectrumCstSPtr(899));
 
 
-    accessor.setPreferredFileReaderType(pappso::MzFormat::brukerTims,
-                                       pappso::FileReaderType::tims_ms2);
+    accessor.setPreferredFileReaderType(pappso::MsDataFormat::brukerTims,
+                                        pappso::FileReaderType::tims_ms2);
 
 
     pappso::MsRunReaderSPtr p_msreader =
@@ -455,8 +556,8 @@ TEST_CASE("Test tims frames", "[timsframe]")
 
     INFO("Test case pappso::FileReaderType::tims start");
 
-    accessor.setPreferredFileReaderType(pappso::MzFormat::brukerTims,
-                                       pappso::FileReaderType::tims);
+    accessor.setPreferredFileReaderType(pappso::MsDataFormat::brukerTims,
+                                        pappso::FileReaderType::tims);
 
 
     class SpectrumCollectionHandler : public SpectrumCollectionHandlerInterface
@@ -526,6 +627,8 @@ TEST_CASE("Test tims frames", "[timsframe]")
     // : 1447.3
     // => avoiding sql query is not really efficient
   }
+
+
   /*
 std::cout << std::endl
   << mass_spectrum.getMassSpectrumCstSPtr()
@@ -581,7 +684,8 @@ polynome :
 #elif USEPAPPSOTREE == 1
 
   std::cout << std::endl
-       << "..:: NO test TIMS TIC chromatogram extraction ::.." << std::endl;
+            << "..:: NO test TIMS TIC chromatogram extraction ::.."
+            << std::endl;
 
 #endif
 }
